import os
import glob
import inspect
import fnmatch

import numpy as np

import matplotlib.pyplot as plt
from netCDF4 import Dataset as ncfile
try:
    import seawater.gibbs as gsw
    import seawater.csiro as csw
except ImportError:
    pass # module doesn't exist, deal with it.

#import alti_tools as atools
from scipy import interpolate
from warnings import warn
from altimetry.tools import recale_limits, in_limits, cumulative_distance, calcul_distance, \
    where_list, \
    cnes_convert, \
    plot_map, \
    get_caller
from collections import OrderedDict


#Additional functions
def load_ncVar(varName, nc=None, **kwargs):
        
        if (nc is None) : raise Exception('No Netcdf file passed')
        
        #Load variable
        var = nc.variables[varName]
        
        var.set_auto_maskandscale(False)
        
        #Load dimensions
        varDim = [str(dim) for dim in var.dimensions]
        missDim=len(varDim) == 0
        if (missDim): warn('No dimension found')
        else : varDimval = [len(nc.dimensions[dimname]) for dimname in varDim]
        
        #Load Attributes
        attrStr=var.__dict__
        
        ind_list = [] #Init index list
        dims = OrderedDict({'_ndims':0}) #Init dimensions
        
        dstr=[]
        shape=()

        #Construct index list
        #looping on variable dimension list
        for vid,vn in enumerate(varDim) :
            
            #No indexation on current dimension
            if not kwargs.has_key(vn) :
                ind_list.append(xrange(varDimval[vid]))
                dims.update({enum[1]:varDimval[enum[0]]})
            
            #Data is indexed along current dimension
            else :
                dumind = kwargs[vn]
#                if not isinstance(dumind,list) : dumind=tuple(dumind)
                if isinstance(dumind,np.ndarray) : dumind=dumind.tolist() #Rq: tolist() can take a very long time to run on large arrays
                if type(dumind) is not list : dumind=[dumind] 
                ind_list.append(dumind)
#                ind_list=(ind_list,dumind)  if len(ind_list) != 0 else (dumind,)
                dims.update({vid:len(dumind)})
        
        #check index list
        sz=[len(i) for i in ind_list]
        
        #find empty dimensions
        if not (where_list([0],sz)[0] == -1 ) : varOut=var[[0]][[]] #np.array(where_list(sz,[0])) == -1
        else :
            varOut=var[ind_list]#.copy() #THIS IS LONG!!
            if var.shape == (1,1) : varOut=varOut.reshape(var.shape)
        
        #Mask it!
        if var.__dict__.has_key('_FillValue') : mask=varOut == var._FillValue
        elif var.__dict__.has_key('missing_value') : mask=varOut == var.missing_value
        else : mask=np.zeros(varOut.shape,dtype='bool')
        
        #Scale it
        #note : we do not use the *= or += operators to force casting to scaling attribute types
        if var.__dict__.has_key('scale') : varOut =varOut * var.scale
        elif var.__dict__.has_key('scale_factor') : varOut = varOut * var.scale_factor
        if var.__dict__.has_key('add_offset') : varOut = varOut + var.add_offset
        
        #Set masks properly
        if isinstance(varOut,np.ndarray) : varOut=np.ma.masked_array(varOut,mask=mask)
        elif isinstance(varOut,np.ma.masked_array) : var.mask=mask
        else : raise 'This data type {} has not been defined - code it!'.format(type(varOut))
        
        #Get attributes
        attrStr=var.__dict__
        attrStr.pop('_FillValue',None) #Remove this attributed as it is overidden
        
        #Append attributes to varOut
        varOut.__dict__.update(attrStr)
        
        #Build up output structure
        outStr={'_dimensions':dims,'data':varOut}
        dims.update({'_ndims':len(dims.keys()[1:])})
        
        return outStr
#            ind_list=[[]] 
    



'''
NCTOOLS
@summary: Netcdf data object, to help loading and writing data
@change Created on 7 sept. 2012
@author: rdussurg
'''

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.pylab as pylab

#from grid.regridding import griddata, xy2xy

from netCDF4 import Dataset as ncfile
import glob
import os
from altimetry.tools import recale, in_limits, where_list, recale_limits, get_caller
#import altimetry.data.alti_tools as atools
from collections import OrderedDict
from warnings import warn

def load_ncVar(varName, nc=None, **kwargs):
        
        if (nc is None) : raise Exception('No Netcdf file passed')
        
        #Load variable
        var = nc.variables[varName]
        
        var.set_auto_maskandscale(False)
        
        #Load dimensions
        varDim = [str(dim) for dim in var.dimensions]
        missDim=len(varDim) == 0
        if (missDim): warn('No dimension found')
        else : varDimval = [len(nc.dimensions[dimname]) for dimname in varDim]
        
        #Load Attributes
        attrStr=var.__dict__
        
        ind_list = [] #Init index list
        dims = OrderedDict({'_ndims':0}) #Init dimensions
        
        dstr=[]
        shape=()

        #Construct index list
        #looping on variable dimension list
        for vid,vn in enumerate(varDim) :
            
            #No indexation on current dimension
            if not kwargs.has_key(vn) :
                dstr=np.append(dstr,':')
                sz=np.long(varDimval[vid])
            
            #Data is indexed along current dimension
            else :
                drange=kwargs[vn]
                if len(drange) == 2 : drange = drange + (1,)
                if nc.variables.has_key(vn) : #Check if current dimension exists
                    dumvar = nc.variables[vn][:]
                else :
                    dumvar = np.arange(len(nc.dimensions[vn]))
                if vn.startswith('lon') : dumvar=recale(dumvar,degrees=True)
                fg=(dumvar >= drange[0]) & (dumvar <= drange[1])
                if fg.sum() == 0 :
                    #retry switrhcing lon/lat
                    dumvar=recale(dumvar,degrees=True)
                    drange=tuple(recale(drange,degrees=True).astype(np.long))
                    fg=(dumvar >= drange[0]) & (dumvar <= drange[1])
                if fg.sum() == 0 :
                    raise IndexError('{0} {1} is not matching given dimensions {2}'.format(vn,(np.nanmin(nc.variables[vn][:]),np.nanmax(nc.variables[vn][:])),drange))
                if len(fg) == 1 :
                    dstr=np.append(dstr,':')
                    sz=1L
                elif len(fg) == 0:
                    sz=0L
                else :
                    dumind=np.arange(varDimval[vid]).compress(fg)
                    bg=dumind[0]
                    en=dumind[-1]+1
                    st=drange[2]
                    dstr=np.append(dstr,'{0}:{1}:{2}'.format(bg,en,st))
                    sz = np.long(np.mod(np.float(en-bg-1)/st,np.float(en-bg)) + 1.)
            
            dims.update({vn:sz})
            shape = shape + (sz,)
#                if isinstance(dumind, np.ndarray) : dumind = dumind.tolist() #Rq: tolist() can take a very long time to run on large arrays
#                if type(dumind) is not list : dumind = [dumind] 
#                ind_list.append(dumind)
#                dims.update({vn:len(dumind)})
        
        #check index list
        sz=[len(i) for i in ind_list]
        
        dstr=','.join(dstr) #invert dimension list for numpy
#        dstr=','.join(dstr[::-1]) #invert dimension list for numpy
        if missDim : cmd = 'varOut = var[:]'
        else : cmd = 'varOut = var[{0}]'.format(dstr)
        exec(cmd)
        
        #find empty variables
#        if not (atools.where_list([0], shape)[0] == -1) : varOut = var[[0]][[]]
#        else : varOut = var[ind_list]
        
        #Mask it!
        if var.__dict__.has_key('_FillValue') :
            fill_value=var._FillValue
            mask = varOut == var._FillValue
        elif var.__dict__.has_key('missing_value') :
            fill_value=var._FillValue
            mask = varOut == var._FillValue
        else :
            fill_value = None
            mask = np.zeros(varOut.shape, dtype='bool')
        
        #Scale it
        #note : we do not use the *= or += operators to force casting to scaling attribute types
        if var.__dict__.has_key('scale') : varOut = varOut * var.scale
        elif var.__dict__.has_key('scale_factor') : varOut = varOut * var.scale_factor
        if var.__dict__.has_key('add_offset') : varOut = varOut + var.add_offset
        
        #Set masks properly
        if isinstance(varOut, np.ndarray) : varOut = np.ma.masked_array(varOut, mask=mask,dtype=varOut.dtype,fill_value=fill_value)
        elif isinstance(varOut, np.ma.masked_array) : var.mask = mask
        else : raise 'This data type {} has not been defined - code it!'.format(type(varOut))
        
        #Update masked data properly
        varOut.data[varOut.mask]=varOut.fill_value
        
        #Switch dimensions 
        if not missDim : varOut=np.transpose(varOut,tuple(range(len(dims.keys()[1:]))[::-1]))
        
        #Build up output structure
        dims.update({'_ndims':len(dims.keys()[1:])})
        outStr = {'_dimensions':dims, 'data':varOut}
        
        #Add variable attributes
        for A in var.__dict__.keys():
            outStr[A]=var.getncattr(A)
        
        return outStr
#            ind_list=[[]] 
    



# -*- coding: utf-8 -*-
import numpy as np
#import alti_tools as atools
import datetime
if __debug__ : import matplotlib.pyplot as plt

def cnes_convert(argin,
                 julian=True,
                 calendar=False,
                 matlab=False,
                 #nasa=False,
                 #string=False,
                 #calformat=False,
                 epoch=None,
                 fromReference=None,
                 verbose=False):
    
    
    try : isVector = len(argin) >= 1 #True if variable is a vector
    except TypeError: isVector=False
    if isVector and (type(argin[0]) == str) and (np.size(argin) != len(argin)): isVector = False #Distinguish between string scalar and arrays
    
    if not isVector : argin=[argin]

    #Get mask if any
    if isinstance(argin, np.ma.masked_array) : maskin=argin.mask
    elif type(argin[0]) == str: maskin=np.zeros(np.shape(argin),dtype=bool)
    else : maskin=~np.isfinite(argin)
    
    #Scalar to vector conversion
    if isinstance(argin, np.ma.masked_array) : argin = argin.tolist(0)
    elif isinstance(argin, np.ndarray) : argin = argin.tolist()
    elif isinstance(argin, list) : pass
    else : raise Exception('Undefined type')
    
    if type(argin[0]) == str : julian = True
    else : calendar = True
    
#    print type(argin[0])
    
    if calendar is True : julian = False
    if julian is True : calendar = False

    if epoch is None : epoch = datetime.datetime(1950, 01, 01)
    if matlab is True : epoch = datetime.datetime(1, 1, 1)
    
    if julian is True :
        if verbose is True : print("julian is true")
#        srlist=[np.array(x.split("/",2) for x in argin,dtype=int)]
        narg = np.size(argin)
        
        strlist  =[x.split("-") for x in argin]
        datelist = []
        for x in strlist :
            if len(x) == 1 : datelist.append(datetime.datetime.strptime(x[0].strip(),"%d/%m/%Y") )
            else : datelist.append(datetime.datetime.strptime('{0}-{1}'.format(x[0].strip(),x[1].strip()),"%d/%m/%Y-%H:%M"))  
#        datelist = [datetime.datetime.strptime(x,"")""]
#        strlist = np.array([x.split("/", 2) for x in argin], dtype=int)
#        datelist = [datetime.datetime(strlist[x, 2], strlist[x, 1], strlist[x, 0]) for x in np.arange(narg)]
        argout=np.array([(datelist[x] - epoch).days for x in np.arange(narg)], dtype=float)
        argout[maskin]=np.NaN if not isinstance(argout,np.ma.masked_array) else np.ma.array(argout.fill_value,mask=True)
        return argout,datelist
    
        
    if calendar is True :
        if verbose is True : print("caldendar is true")
        datelist = [epoch.toordinal() + x for x in  argin]
        if verbose is True : print(datelist)
#        days = [np.int(x) for x in datelist]
#        decim = np.array(datelist) - np.array(days)
#        return [datetime.date.fromordinal(y) for y in datelist]
        a=datetime.datetime.fromordinal(int(datelist[0]))
        dateObjList=np.array([datetime.datetime.fromordinal(int(y)) + datetime.timedelta(seconds=(y - np.floor(y))*86400.0) for y in datelist])
        dateStr=np.array([Obj.strftime('%d/%m/%Y') for Obj in dateObjList])
        dateStr[maskin]=None if not isinstance(dateStr,np.ma.masked_array) else np.ma.array(dateStr.fill_value,mask=True)
        dateObjList[maskin]=None if not isinstance(dateStr,np.ma.masked_array) else np.ma.array(dateStr.fill_value,mask=True)
        return dateStr.tolist(),dateObjList.tolist()
#        dd,mm,yyyy=np.array(argin.split("/",2),dtype=int)
#        varout = np.array((datetime.datetime(yyyy,mm,dd) - epoch).days,dtype=float)
        

def date2seas( date, seas=['DJF','MAM','JJA','SON'], shift=1):
    """
    #===============================================================================
    # ;+
    # ;
    # ; CDO_date2seas : provides the equivalent CDO season to a julian date array
    # ;
    # ; @param date {in}{required}{type=NUMERIC} julian date (CNES days)
    # ; @keyword seas {in}{optional}{type=STRING} seasons vector !! CAUTION !! This <br />
    # ;   must be accompanied by the SHIFT variable
    # ; @keyword shift {in}{optional}{type=NUMERIC} Backward in time offset of the 
    # ;   first element of SEAS vector in number of months from January <br />
    # ;   (e.g. Decembre -> 1, Novembre -> 2, etc... )
    # ; 
    # ;
    # ;-
    #===============================================================================
    """
        

    corr_months=np.roll(np.arange(12).reshape((4,3))+1,shift)
    
    nt = len(date)
    outvec=(cnes_convert(date))[1]
    month=np.array([d.month for d in outvec])
    outseas_ind=-np.ones(nt)
#    outseas=np.repeat('   ',nt)
    
    for i,m in enumerate(month) : outseas_ind[i] = int(np.where(m == corr_months)[0])
    outseas =[seas[int(i)] for i in outseas_ind]
    
    return outseas, outseas_ind

def cnes2modis(cnes_date,YYYYDDDHHMM=None,YYYYDDD=True):
    """
    #+
    # CNES2MODIS : Convert CNES julian days to MODIS data format (YYYYDDD)
    # 
    # @author: Renaud DUSSURGET (LEGOS/CTOH)
    # @history: Created by RD on 2/12/2011
    #           Adapted to Python on 29/10/2012 by RD (now at LER PAC/IFREMER)
    #
    #-
    """

    #Setup defaults
    if YYYYDDDHHMM is None : YYYYDDDHHMM = False
    if YYYYDDD : YYYYDDDHHMM = False
    if YYYYDDDHHMM : YYYYDDD = False

    str,obj=cnes_convert(cnes_date)
    dat=[o.strftime("%Y%j%H%M") for o in obj]
    
    return dat

def modis2cnes(modis_date):
    """
    #+
    # MODIS2CNES : Convert MODIS date format to CNES JULIAN DAYS (YYYYDDD or YYYYDDDHHMM)
    # 
    # @author: Renaud DUSSURGET (LER PAC/IFREMER)
    # @history: Created by RD on 29/10/2012
    #
    #-
    """

    if not isinstance(modis_date,list) : modis_date=[modis_date]

    if len(modis_date[0]) ==  7 :
        obj=[datetime.datetime.strptime(d,"%Y%j") for d in modis_date] 
    else :
        obj=[datetime.datetime.strptime(d,"%Y%j%H%M") for d in modis_date]
    
#    str,obj=cnes_convert(modis_date)
    dat=[o.strftime("%d/%m/%Y-%H:%M") for o in obj]
    
    return cnes_convert(dat)





import sys

try:
    import numpy
    have_numpy=True
except:
    have_numpy=False

try:
    import scipy
    from scipy.interpolate import interp1d
    import scipy.weave
    have_scipy=True
except:
    have_scipy=False





def histogram(data, bins=None, binsize=1., min=None, max=None, rev=False, use_weave=True, verbose=0):
    """
    Similar to IDL histogram.

    For reverse indices, the fast version uses weave from scipy. This is the
    default.  If scipy is not available a slower version is used.

    """

    if not have_scipy:
        use_weave=False
     
    dowhere=False
    s = data.argsort()
    if min is not None:
        dmin = min
        dowhere=True
    else:
        dmin = data[s[0]]

    if max is not None:
        dmax = max
        dowhere=True
    else:
        dmax = data[s[-1]]

    bsize = float(binsize)
#    except TypeError :
#        print '[Warning] TypeError reached - type {0}'.format(type(binsize[0]))
#        bsize = binsize
#    finally : pass

    if dowhere:
        # where will preserve order, so subscript with s
        w,=numpy.where( (data[s] >= dmin) & (data[s] <= dmax) )
        if w.size == 0:
            raise ValueError("No data in specified min/max range\n")
        s = s[w]

    nbin = numpy.int64( (dmax-dmin)/bsize ) + 1

    if bins is not None :
        s = bins
        nbin = len(s)

    if rev:
        revsize = s.size + nbin+1
        revind = numpy.zeros(revsize, dtype='i8')
    else:
        # this is just a dummy variable
        revind=numpy.zeros(1, dtype='i8')
    hist = numpy.zeros(nbin, dtype='i8')

    # populate the array from nbin+1:nbin+1+s.size
    # with the sort indices.  Simultaneosly record bin
    # edges at the beginning of reverse indices

    if use_weave:
        _weave_dohist(data, s, bsize, hist, revind, dorev=rev,verbose=verbose)
    else:
        _dohist(data, s, bsize, hist, revind, dorev=rev)

    if rev:
        return hist, revind
    else:
        return hist


def _weave_dohist(data, s, binsize, hist, rev, dorev=False, verbose=0):

    if dorev:
        dorev=1
    else:
        dorev=0
    """
    Weave version of histogram with reverse_indices

    s is an index into data, sorted and possibly a subset
    """
    code = """

    int64_t nbin = hist.size();
    int64_t binnum_old = -1;

    // index of minimum value
    int64_t imin = s(0);
    for (int64_t i=0; i<s.size(); i++) {

        int64_t offset = i+nbin+1;
        int64_t data_index = s(i);


        if (dorev) {
            rev(offset) = data_index;
        }

        int64_t binnum = (int64_t) ( (data(data_index)-data(imin))/binsize);

        if (binnum >= 0 && binnum < nbin) {
            if (dorev && (binnum > binnum_old) ) {
                int64_t tbin = binnum_old + 1;
                while (tbin <= binnum) {
                    rev(tbin) = offset;
                    tbin++;
                }
            }
            hist(binnum) = hist(binnum) + 1;
            binnum_old = binnum;
        }
    }

    int64_t tbin = binnum_old + 1;
    while (tbin <= nbin) {
        if (dorev) {
            rev(tbin) = rev.size();
        }
        tbin++;
    }

    """

    scipy.weave.inline(code, ['data','s','binsize','hist','rev','dorev'],
                       type_converters = scipy.weave.converters.blitz, verbose=verbose)
    return


def _dohist(data, s, binsize, hist, revind, dorev=False):

    nbin=hist.size
    offset = nbin+1
    i=0
    binnum_old = -1

    dmin = data[s[0]]
    while i < s.size:
        data_index = s[i]
        if dorev:
            revind[offset] = data_index

        val = data[data_index]

        binnum = numpy.int64( (val-dmin)/binsize )
        #print 'binnum:',binnum,' binnum old:',binnum_old, 'val:',val
        if binnum >= 0 and binnum < nbin:
        #if binnum >= 0:
            if binnum > binnum_old:
                tbin = binnum_old + 1
                while tbin <= binnum:
                    if dorev:
                        revind[tbin] = offset
                        #print '\t\trevind[%d] = %d' % (tbin,offset)
                    tbin += 1

            hist[binnum] += 1
            binnum_old = binnum
        #print 'rev:',revind[binnum]

        i += 1
        offset += 1

    if dorev:
        # Fill in the last ones
        tbin = binnum_old + 1
        while tbin <= nbin:
            revind[tbin] = revind.size
            tbin += 1





def histogram2d(x, y, 
                nx=None, 
                ny=None, 
                xbin=None, 
                ybin=None, 
                xmin=None, 
                xmax=None, 
                ymin=None, 
                ymax=None, 
                rev=False):

    if x.size != y.size:
        raise ValueError("x and y must be the same size")

    # binsizes will take precedence
    dobinsizes=False
    donbin=False
    if (xbin != None) and (ybin != None):
        dobinsizes=True
    elif (nx != None) or (ny != None):
        donbin=True
    else:
        raise ValueError("Enter either nx,ny or xbin,ybin")

    if xmin is None:
        xmin=x.min()
    if xmax is None:
        xmax=x.max()
    if ymin is None:
        ymin=y.min()
    if ymax is None:
        ymax=y.max()

    w, = numpy.where( (x >= xmin) & (x <= xmax) & (y >= ymin) & (y <= ymax) )
    if w.size == 0:
        raise ValueError("No data in specified min/max range\n")

    if dobinsizes:
        # determine nx,ny from binsizes
        nx = numpy.int64(  (xmax-xmin)/xbin ) + 1
        ny = numpy.int64(  (ymax-ymin)/ybin ) + 1

    xind=numpy.floor((x[w]-xmin)*(nx/(xmax-xmin)))
    yind=numpy.floor((y[w]-ymin)*(ny/(ymax-ymin)))

    ind=xind+nx*yind


    hist=histogram(ind, min=0l, max=nx*ny-1, rev=rev)
    if rev:
        hist, revind=hist
        hist = hist.reshape(nx,ny)
        return hist, revind
    else:
        hist = hist.reshape(nx,ny)
        return hist


def wmom(arrin, weightsin, inputmean=None, calcerr=False, sdev=False):
    """
    NAME:
      wmom()
      
    PURPOSE:
      Calculate the weighted mean, error, and optionally standard deviation
      of an input array.

    CALLING SEQUENCE:
     wmean,werr = wmom(arr, weights, inputmean=None, calcerr=False, sdev=False)
    
    INPUTS:
      arr: A numpy array or a sequence that can be converted.
      weights: A set of weights for each elements in array.
    OPTIONAL INPUTS:
      inputmean: An input mean value, around which them mean is calculated.
      calcerr=False: Calculate the weighted error.  By default the error
        is calculated as 1/sqrt( weights.sum() ).  In this case it is
        calculated as sqrt( (w**2 * (arr-mean)**2).sum() )/weights.sum()
      sdev=False: If True, also return the weighted standard deviation 
        as a third element in the tuple.

    OUTPUTS:
      wmean, werr: A tuple of the weighted mean and error. If sdev=True the
         tuple will also contain sdev: wmean,werr,wsdev

    REVISION HISTORY:
      Converted from IDL: 2006-10-23. Erin Sheldon, NYU

   """
    from numpy import float64
    
    # no copy made if they are already arrays
    arr = numpy.array(arrin, ndmin=1, copy=False)
    weights = numpy.array(weightsin, ndmin=1, copy=False)
    
    # Weights is forced to be type double. All resulting calculations
    # will also be double
    if weights.dtype != float64:
        weights = numpy.array(weights, dtype=float64)
  
    wtot = weights.sum()
        
    # user has input a mean value
    if inputmean is None:
        wmean = ( weights*arr ).sum()/wtot
    else:
        wmean=float(inputmean)

    # how should error be calculated?
    if calcerr:
        werr2 = ( weights**2 * (arr-wmean)**2 ).sum()
        werr = numpy.sqrt( werr2 )/wtot
    else:
        werr = 1.0/numpy.sqrt(wtot)

    # should output include the weighted standard deviation?
    if sdev:
        wvar = ( weights*(arr-wmean)**2 ).sum()/wtot
        wsdev = numpy.sqrt(wvar)
        return wmean,werr,wsdev
    else:
        return wmean,werr




def sigma_clip(arrin, niter=4, nsig=4, extra={}, verbose=False):
    """
    NAME:
      sigma_clip()
      
    PURPOSE:
      Calculate the mean/stdev of an array with sigma clipping. Iterate
      niter times, removing elements that are outside nsig, and recalculating
      mean/stdev.

    CALLING SEQUENCE:
      mean,stdev = sigma_clip(arr, niter=4, nsig=4, extra={})
    
    INPUTS:
      arr: A numpy array or a sequence that can be converted.

    OPTIONAL INPUTS:
      niter: number of iterations, defaults to 4
      nsig: number of sigma, defaults to 4

    OUTPUTS:
      mean,stdev: A tuple containing mean and standard deviation.
    OPTIONAL OUTPUTS
      extra={}: Dictionary containing the array of used indices in
         extra['index']

    REVISION HISTORY:
      Converted from IDL: 2006-10-23. Erin Sheldon, NYU

   """
    arr = numpy.array(arrin, ndmin=1, copy=False)

    index = numpy.arange( arr.size )

    for i in numpy.arange(niter):
        m = arr[index].mean()
        s = arr[index].std()

        if verbose:
            sys.stdout.write('iter %s\tnuse: %s\tmean %s\tstdev %s\n' % \
                (i+1, index.size,m,s))

        clip = nsig*s

        w, = numpy.where( (numpy.abs(arr[index]) - m) < clip )

        if w.size == 0:
            sys.stderr.write("nsig too small. Everything clipped on iteration %d" % i+1)
            return m,s


        index = index[w]

    # Calculate final stats
    amean = arr[index].mean()
    asig = arr[index].std()

    extra['index'] = index
    return amean, asig
     




'''
CPT_READER : Read cpt palette and returns a segmented color dictionary for use in matplotlib

@author: David Huard (david.huard-Re5JQEeQqe8AvxtiuMwx3w@public.gmane.org)
         Renaud Dussurget, LER PAC/IFREMER (RD, renaud.dussurget@ifremer.fr)
@change: Created by DH in February 2006
    ported into PyValid on 21 janv. 2013 (RD)
'''
from scipy import zeros, linspace, shape, float32 as Float, concatenate
import numpy as np
import matplotlib.colors
import glob, os
from altimetry import defaults


def cpt2seg(file_name, sym=False, discrete=False):
    """Reads a .cpt palette and returns a segmented colormap.

    sym : If True, the returned colormap contains the palette and a mirrored copy.
    For example, a blue-red-green palette would return a blue-red-green-green-red-blue colormap.

    discrete : If true, the returned colormap has a fixed number of uniform colors.
    That is, colors are not interpolated to form a continuous range.

    Example :
    >>> _palette_data = cpt2seg('palette.cpt')
    >>> palette = matplotlib.colors.LinearSegmentedColormap('palette', _palette_data, 100)
    >>> imshow(X, cmap=palette)
    """
   
   
    dic = {}
#    io
#    f = scipy.io.open(file_name, 'r')
#    rgb = f.read_array(f)
    
    #Check flags:
#    with open(file_name) as f:
#        content = f.readlines()
#        header=np.where(np.array([c.startswith('#') for c in content]))[0][-1]
#        footer=np.where(np.array([c[0].isalpha() for c in content]))[0][0]
    
    rgb = np.genfromtxt(file_name,comments='#',invalid_raise=False)
#    rgb = np.genfromtxt(file_name)
    rgb = rgb/255.
    s = shape(rgb)
    colors = ['red', 'green', 'blue']
    for c in colors:
        i = colors.index(c)
        x = rgb[:, i+1]

        if discrete:
            if sym:
                dic[c] = zeros((2*s[0]+1, 3), dtype=Float)
                dic[c][:,0] = linspace(0,1,2*s[0]+1)
                vec = concatenate((x ,x[::-1]))
            else:
                dic[c] = zeros((s[0]+1, 3), dtype=Float)
                dic[c][:,0] = linspace(0,1,s[0]+1)
                vec = x
            dic[c][1:, 1] = vec
            dic[c][:-1,2] = vec
               
        else:
            if sym:
                dic[c] = zeros((2*s[0], 3), dtype=Float)
                dic[c][:,0] = linspace(0,1,2*s[0])
                vec = concatenate((x ,x[::-1]))
            else:
                dic[c] = zeros((s[0], 3), dtype=Float)
                dic[c][:,0] = linspace(0,1,s[0])
                vec = x
            dic[c][:, 1] = vec
            dic[c][:, 2] = vec
   
    return dic

def revert_cpt(cptdata):
    outStr=cptdata.copy()
    for k in outStr.keys() : outStr[k][:,1:]=outStr[k][::-1,1:]
    
    return outStr

def get_cmap(cmap_name,revert=False,N=256):
    
    cpt_dir=defaults.cptDir
    
    ls = glob.glob(cpt_dir + cmap_name + '.cpt')
    if len(ls) == 0 : raise Exception('file {0} do not exists'.format(os.path.basename(cpt_dir + cmap_name + '.cpt')))
    cptdata=cpt2seg(ls[0])
    if revert : cptdata=revert_cpt(cptdata)
    cmap = matplotlib.colors.LinearSegmentedColormap ('palette', cptdata, N)
    
    return cmap



# -*- coding: utf-8 -*-
import os
from warnings import warn
import matplotlib.pyplot as plt
import matplotlib.pylab as pylab
from mpl_toolkits.basemap import Basemap
import numpy as np
from altimetry import defaults
from altimetry.tools import track_orient, in_limits, calcul_distance, recale, nctools as nc
from collections import OrderedDict
try : from scipy import io
except ImportError :
    warn("[WARNING:%s] module scipy not found" % __name__)

class plot_map(Basemap):
    def __init__(self,*args,**kwargs):
#                 resolution='i',
#                 xoffset=.05,
#                 yoffset=.05,
#                 length=50.,
#                 scale_lon=None,
#                 scale_lat=None,
#                 s=25,
#                 c='k',
#                 limit=None,cp -rf /home/du2_dev/dev/OCTANT/incl/*
#                 edgecolor='none'):
    
        #Startup sequence
        #################
        
        c = kwargs.pop('c', 'k')
        s = kwargs.pop('s', 25)
        edgecolor = kwargs.pop('edgecolor', 'none')
        limit = kwargs.pop('limit', None)
        resolution = kwargs.pop('resolution', 'i')
        projection = kwargs.pop('projection', 'merc')
        xoffset = kwargs.pop('xoffset', 0.05)
        yoffset = kwargs.pop('yoffset', 0.05)
        length = kwargs.pop('length', 50.)
        scale_lon = kwargs.pop('scale_lon', None)
        scale_lat = kwargs.pop('scale_lat', None)
        
        lon=[]
        lat=[]
        z=[]
        if len(args) >= 2 :
            lon=args[0]
            lat=args[1]
        if len(args) == 3:
            z=args[2]
        if np.size(lon) == 1: c='o'+c
        
        #Force fields
        lon=np.array(lon)
        lat=np.array(lat)
        
        #Load default values (requested by Basemap.__init__()
        if limit is None : limit=np.array([lat.min() - 0.5, lon.min() - 0.5, lat.max() + 0.5, lon.max() + 0.5])
        else : limit=np.array(limit)
        
        clon = limit[[1, 3]].mean()
        clat = limit[[0, 2]].mean()
        
        #Init sequence
        ##############
        
        
        # Init Basemap class
        Basemap.__init__(self,projection=projection, resolution=resolution, llcrnrlat=limit[0], llcrnrlon=limit[1], urcrnrlat=limit[2], urcrnrlon=limit[3], lat_0=clat, lon_0=clon)
        
        #Set map characteristics fields
        self.limit=limit
        self.clon=clon
        self.clat=clat
        self.mcenter = self(self.clon, self.clat) #central position
        cparallel = np.array(self(self.limit[[1, 3]], [self.clat, self.clat])) #Get central paralell and meridian coordinates
        cmeridian = np.array(self(self.limit[[0, 2]], [self.clon, self.clon]))
        self.mllcorner = self(self.limit[1], self.limit[0])
        self.width = np.sqrt(np.diff(cparallel[:, 0]) ** 2 + np.diff(cparallel[:, 1]))
        self.heigth = np.sqrt(np.diff(cmeridian[:, 0]) ** 2 + np.diff(cmeridian[:, 1]))
        self.mxoffset = xoffset * self.width
        self.myoffset = yoffset * self.heigth
        
        if scale_lon is None :
            self.scale_lon = 0.9 * self.width + self.mllcorner[0]
            self.scale_lat = 0.1 * self.heigth + self.mllcorner[1]
        else :
            self.scale_lon = self(scale_lon, self.clat)
            self.scale_lat = self(self.clon, scale_lat)
        
        #Set plot characterisitics fields
        self.resolution=resolution
        self.length=length

        #Draw map
        if np.size(z) > 1 : self.scatter(lon,lat,z,s=s,edgecolor=edgecolor,**kwargs)
        elif np.size(z) == 1 :
            if z != 0 : self.plot(lon,lat,z,s=s,edgecolor=edgecolor,**kwargs)
            else :
                if lon != 0 : self.plot(lon,lat,c,s=s,**kwargs)
        elif np.size(z) == 0 :
            if lon != 0 : self.plot(lon,lat,c,s=s,**kwargs)
    
        self.setup_map(**kwargs)
#        if np.size(z) > 1 : return self.setup_map(lon, lat, z, s=s, edgecolor=edgecolor)
#        elif np.size(z) != 0 :
#            if z != 0 :  return self.setup_map(lon, lat, z, s=s, edgecolor=edgecolor)
#        elif np.size(z) == 0 :
#            self.setup_map()
        
#        self.show() #RQ : Show erase everything!!       
#        self.m.drawmapscale(self.scale_lon, self.scale_lat, self.mcenter[0], self.mcenter[1], length=self.length)

    
    #Redefined methods
    ##################
    def scatter(self,*args,**kwargs):

        mlon, mlat = self(np.array(args[0]),np.array(args[1]))
        
        if len(args) == 3 :
            c = np.array(args[2])
            if 'c' in kwargs :del kwargs['c']
        
        if 'edgecolor' in kwargs :
            edgecolor=kwargs['edgecolor']
            del kwargs['edgecolor']
        else : edgecolor = 'none'

        return Basemap.scatter(self,mlon,mlat,c=c,edgecolor=edgecolor,**kwargs)
#        self.colorbar(orientation='horizontal')
    
    def pcolormesh(self,*args,**kwargs):
        
        if len(args) == 1 : raise Exception('Lon/Lat arguments must be provided')
        
        lon=np.array(args[0]) 
        lat=np.array(args[1])
        
        if lon.ndim != 2 :
            lon,lat=np.meshgrid(lon,lat)
            lon=lon.transpose()
            lat=lat.transpose()
        
        mlon, mlat = self(lon,lat)
        image=np.ma.masked_array(args[2]) if not isinstance(args[2], np.ma.masked_array) else args[2]
#        image=np.ma.masked_array(args[2])
        return Basemap.pcolormesh(self,mlon,mlat,image,**kwargs)
    
    def contour(self,*args,**kwargs):
        
        lon=np.array(args[0]) 
        lat=np.array(args[1])
        
        if lon.ndim != 2 :
            lon,lat=np.meshgrid(lon,lat)
            lon=lon.transpose()
            lat=lat.transpose()
        
        mlon, mlat = self(lon,lat)
        
        if len(args) == 3 : return Basemap.contour(self,mlon,mlat,args[2],**kwargs)
        else : return Basemap.contour(self,mlon,mlat,args[2],args[3],**kwargs)
        
    def contourf(self,*args,**kwargs):
        
        lon=np.array(args[0]) 
        lat=np.array(args[1])
        
        if lon.ndim != 2 :
            lon,lat=np.meshgrid(lon,lat)
            lon=lon.transpose()
            lat=lat.transpose()
        
        mlon, mlat = self(lon,lat)
        
        if len(args) == 3 : return Basemap.contourf(self,mlon,mlat,args[2],**kwargs)
        else : return Basemap.contourf(self,mlon,mlat,args[2],args[3],**kwargs)
    
#    def clabel(self,*args,**kwargs):
#        Basemap.cl
    
    def drawmapscale(self, lon, lat, lon0, lat0, length, barstyle='simple', 
        units='km', fontsize=9, yoffset=None, labelstyle='simple', 
        fontcolor='k', fillcolor1='w', fillcolor2='k', ax=None, 
        format='%d', zorder=None):
        
#        x1,y1 = 0.3*m.xmax, 0.25*m.ymax
#        mlon,mlat = self(lon,lat,inverse=True)
#        mlon0,mlat0 = self(lon0,lat0,inverse=True)
        return Basemap.drawmapscale(self, lon, lat, lon0, lat0, length, barstyle=barstyle, units=units, fontsize=fontsize, yoffset=yoffset, labelstyle=labelstyle, fontcolor=fontcolor, fillcolor1=fillcolor1, fillcolor2=fillcolor2, ax=ax, format=format, zorder=zorder)
    
    def arrow(self,*args,**kwargs):
        lon,lat=self(args[0],args[1])
        dx=args[2]
        dy=args[3]
        ax=plt.gca()
        rg=np.arange(np.size(lon))
        if rg.max() > 0:
            for i in rg : ax.arrow(lon[i], lat[i], dx[i], dy[i], **kwargs)
        else : ax.arrow(lon, lat, dx, dy, **kwargs)
    
    def across_track_arrow(self,lon,lat,z, scale=None, ref=None, angle=0.0, **kwargs):
        
        if scale is None : scale = calcul_distance(self.clat,self.clon,self.clat,self.clon+1)*1e3
        else : scale *= calcul_distance(self.clat,self.clon,self.clat,self.clon+1)*1e3
        
        if ref is None : sense,orient = track_orient(lon, lat, orient=True)
        else :
            sense = True
            orient = (np.pi/2.)
        
#        if (not track_orient(lon,lat)) : z*=-1    
        
        across_track = orient + angle - (np.pi/2.) if sense else orient + angle + (np.pi/2.)

        if isinstance(z, np.ma.masked_array) : z = z.data

        dx = scale * z * np.cos(across_track)
        dy = scale * z * np.sin(across_track)
        
        if ref is None : self.arrow(lon,lat,dx,dy,**kwargs)
        else :
            reflon = (self.lonmax - self.lonmin)*0.80 + self.lonmin
            reflat = (self.latmax - self.latmin)*0.15 + self.latmin
            reflatarr = (self.latmax - self.latmin)*0.2 + self.latmin
            self.arrow(reflon,reflatarr,scale*ref,0.0,**kwargs)
            if type(ref) == bool : self.text(reflon, reflat, '{0:.1f} m/s'.format(z[0]))
            else : self.text(reflon, reflat, '{0:.1f} m/s'.format(ref))
    
    def quiver(self,*args,**kwargs):
        lon=np.array(args[0]) 
        lat=np.array(args[1])
        
        if lon.ndim != 2 :
            glon,glat=np.meshgrid(lon,lat)
            glon=glon#.transpose()
            glat=glat#.transpose()
        else :
            glon=lon.copy()
            glat=lat.copy()
        
        reference=kwargs.pop('reference',None)
        
        mlon, mlat = self(glon,glat)
        
        u=np.ma.masked_array(args[2]) if not isinstance(args[2], np.ma.masked_array) else args[2]
        v=np.ma.masked_array(args[3]) if not isinstance(args[3], np.ma.masked_array) else args[3]
        
        q=Basemap.quiver(self,mlon,mlat,u,v,**kwargs)
        
        #Refererence argument has shape [lon,lat,value,title] or [lon,lat] or True/False 
        if reference is not None :
            
            #If
            if len(reference) == 1 :
                x=self.limit[1]+(self.limit[3]-self.limit[1])*0.2
                y=self.limit[1]+(self.limit[3]-self.limit[1])*0.8
                reference=[x,y]
            
            x=reference[0]
            y=reference[1]
                
            if len(reference) == 2 :
                value=np.mean(np.sqrt(u**2+v**2))
                title="{0}".format(value)
            else :
                value=reference[2]
                title=reference[3]
            
            self.quiverkey(q,x,y,value,title)
            
        return q  
    
    def quiverkey(self,*args,**kwargs):
        
        obj=args[0]
        lon=args[1] 
        lat=args[2]
        val=args[3]
        leg=args[4]
        
        coordinates=kwargs.pop('coordinates','data')
        
        mlon, mlat = self(lon,lat)
        
        return plt.quiverkey(obj,mlon,mlat,val,leg,coordinates=coordinates,**kwargs)  
    
    #Following is not working
    def plot(self,*args,**kwargs):
        
        mlon, mlat = self(args[0],args[1])
        
        #Get marker color
       
            
        #override if passed directly through 3rd argument
        if len(args) == 3 :
            c = args[2]
        else :
            if 'c' in kwargs :
                c = kwargs['c']
                del kwargs['c']
            else : c = '.k'
            
        #Override plot ms keyword - becomes the same syntax as scatter
        if 's' in kwargs :
            kwargs['ms'] = kwargs['s']
            del kwargs['s']
        
        if isinstance(c,str) :
            return Basemap.plot(self,mlon,mlat,c,**kwargs)
#        self.colorbar(orientation='horizontal')
    
#    def plot(self,lon,lat,
#             s=25,
#             c='k',
#             edgecolors=None):
##             *args):
#
##        if len(args) == 1 : 
##        z=args[0]
#                
#        mlon, mlat = self(lon, lat)
##        if c is None : 
##        if len(args) == 1 :self.m.scatter(mlon, mlat, s=s, c=z, edgecolors=edgecolors)
##        else : self.m.scatter(mlon, mlat, s=s, c=c, edgecolors=edgecolors)
#        self.scatter(mlon, mlat, c=c, s=s, edgecolors=edgecolors)
        
#        if not hasattr(self, 'cbar') and len(args) == 1 : self.cbar = plt.colorbar(orientation='horizontal')
    
    def get_cursor(self,*args):
        c= zip(*(pylab.ginput(n=0,mouse_add=1, mouse_pop=2, mouse_stop=3)))
        lonpt,latpt = self(c[0],c[1],inverse=True)
        for enum in enumerate(zip(*(lonpt,latpt))):
            dst=calcul_distance(enum[1][1],enum[1][0],args[0],args[1])
            i=np.argmin(dst)
            xdum = args[0][i]
            ydum = args[1][i]
            if len(args) == 3 : zdum = args[2][i]
            if enum[0] == 0 :
                xout=xdum
                yout=ydum
                if len(args) == 3 : zout = zdum
            else :
                xout=np.append(xout,xdum)
                yout=np.append(yout,ydum)
                if len(args) == 3 : zout = np.append(zout,zdum)
        if len(args) == 3 : return xout, yout, zout
        else : return xout, yout
                
    def get_cursor_id(self,x,y):
        c= zip(*(pylab.ginput(n=0,mouse_add=1, mouse_pop=2, mouse_stop=3)))
        lonpt,latpt = self(c[0],c[1],inverse=True)
        for enum in enumerate(zip(*(lonpt,latpt))):
            dst=calcul_distance(enum[1][1],enum[1][0],x,y)
            if enum[0] == 0 : i=[np.argmin(dst).tolist()]
            else : i.append(np.argmin(dst))
        return i
    
    def figure(self,*args,**kwargs):
        return plt.figure(*args,**kwargs)
        
    def legend(self,*args,**kwargs):
        plt.legend(*args,**kwargs)
        
    def text(self,lon, lat, string,**kwargs):
        mlon,mlat=self(lon,lat)
        return plt.text(mlon,mlat,string,**kwargs)
    
    def title(self,s,**kwargs):
        plt.title(s,**kwargs)
    
    def colorbar(self,*args,**kwargs):
        label = kwargs.pop('label',None)
        cbar=plt.colorbar(*args,**kwargs)
        if label is not None : cbar.set_label(label)
        return cbar
        
    def setup_map(self,londel=1.,latdel=1.,*args,**kwargs):
        """
        SETUP_MAP : Draw the map and coastlines.
        
        @note: To be compliant with self.legend() -> setup_map() must be called after legend
        """
        
        s = kwargs['s'] if kwargs.has_key('s') else 25
        edgecolor = kwargs['edgecolor'] if kwargs.has_key('edgecolor') else 'none'
        bathy = kwargs.get('bathy',not kwargs.get('nobathy',False))
        
        #Plot Sequence
        ##############
        self.drawcoastlines()
        if bathy : cs=self.drawbathy(**kwargs)
        
        self.drawparallels(np.arange(np.floor(self.limit[0]), np.ceil(self.limit[2]), latdel), labels=[1, 0, 0, 0])
        self.drawmeridians(np.arange(np.floor(self.limit[1]), np.ceil(self.limit[3]), londel), labels=[0, 0, 0, 1])
                
#        if len(args) == 3 :
#            lon=args[0]
#            lat=args[1]
#            z=args[2]
#            if np.size(z) > 1 : self.scatter(lon,lat,z,s=s,edgecolor=edgecolor)
#            else : 
#                if type(z) != type(str()) : z = '.k'
#                self.plot(lon,lat,z,ms=s)*
#        if len(args) == 2 :
#            lon=args[0]
#            lat=args[1]
#            if np.size(z) > 1 : self.plot(lon,lat,z,s=s,edgecolor=edgecolor)
#            else : 
#                if type(z) != type(str()) : z = '.k'
#                self.plot(lon,lat,z,ms=s)
        return cs if bathy else None
    
    def drawbathy(self,fname=None,V=[-250,-2000],Nlevels=None,step=None,colors='k',linestyles='dotted', linewidths=2,**kwargs):

        #If 2 points within, go for menor        
        if np.sum(in_limits(self.limit[[1,3]],self.limit[[0,2]],[35,0,45,25])[1]) ==2 :
            bathy=defaults.menor
        else : bathy = defaults.etopo
        
        if os.path.exists(bathy.path) :
            
            if fname is None :
                bat=load_bathy(bathy=bathy,limit=self.limit)
            else :
                bat=load_bathy(fname=fname,limit=self.limit)
            
            glat,glon=np.meshgrid(np.squeeze(bat['lat']), np.squeeze(bat['lon']))
            
            mn=0.01
            mx=np.nanmin(bat['Z'].data)
            if Nlevels is not None :
                if step is None : step = (mx - mn) / Nlevels
                V = np.arange(mn,mx,step,dtype=np.float)
            
            return self.contour(glon,glat,bat['Z'],V,colors=colors,linestyles=linestyles,linewidths=linewidths,**kwargs)
        else :
	    return self.contour(np.array(self.limit[[1,3]]),np.array(self.limit[[0,2]]),np.ones((2,2),dtype=float),np.array([1.,1.]),colors=colors,linestyles=linestyles,linewidths=linewidths,**kwargs)
        
        
        
    def show(self):
        plt.show()
        
    def savefig(self,*args,**kwargs):
        plt.savefig(*args,**kwargs)
    
#    def set_local(self,VarName,VarValue):
#        if VarName not in locals() : exec('self.'+VarName+' = VarValue')

def load_bathy(bathy='MENOR',**kwargs):
    
    if not bathy.set : Warning('Bathymetry not set : check your altimetry.config module')
    
    if bathy.ext == '.nc' :
        ncf=nc.nc()
        attr=ncf.attributes(bathy.path)
        convetion=attr[[k for k in attr if k.upper().startswith('CONVENTION')][0]].upper()
        
        if len(convetion) == 0 : raise Exception('Unknown NetCDF format') 
    
        if convetion == 'COARDS' :       
            if kwargs.has_key('lon') : kwargs['x']=recale(kwargs.pop('lon'),degrees=True)
            if kwargs.has_key('lat') : kwargs['y']=kwargs.pop('lat')
            if kwargs.has_key('limit') :
                limit=kwargs.pop('limit')
                kwargs['x']=tuple(recale(limit[[1,3]],degrees=True))
                kwargs['y']=tuple(limit[[0,2]])
            str=ncf.read(bathy.path,**kwargs)
            str['lon']=np.ma.array(recale(str.pop('x')['data'],degrees=True))
            str['lat']=np.ma.array(str.pop('y')['data'])
            str['Z']=np.ma.array(str.pop('z')['data'])
            return str
            
        else : raise Exception('Unknown NetCDF convetion {0}'.format(convetion)) 
    
    elif bathy.ext == '.mat' :
        
        m_dict=OrderedDict( (('lon_menor',[]), ('lat_menor',[]), ('H0',[])) )
        str=io.loadmat(bathy.path,m_dict)
        
        str['lon']=np.ma.array(np.squeeze(str['lon_menor']))
        str['lat']=np.ma.array(np.squeeze(str['lat_menor']))
        str['Z']=np.ma.array(str['H0']).transpose()
        
        del str['lon_menor']
        del str['lat_menor']
        del str['H0']
    
        return str
    else : raise Exception('Unknown bathymetry file extension ({0})'.bathy.ext) 



# -*- coding: utf-8 -*-
import threading
import datetime
import numpy as np
import Queue

#import alti_tools as atools


##Global variables (rq: global keyword should be called by a function to set/modify global vars
#global x; x=10000L
#global y; y=2500L
#
#global A; A=np.matrix(np.repeat(np.NaN,x*y).reshape((x,y)))
#global map; map = np.arange(x)


#Init global variables
#dist_matrix=np.empty((0,0),dtype=np.float64)
rt = 6378.137 #Earth radius (km)
verbose = 0

#Computation thread
####################
#Queued thread
##############
class ThreadClass(threading.Thread):
    
    #We override the __init__ method
    def __init__(self, input_q,indices_q,result_q, alat, alon, blat, blon):
        threading.Thread.__init__(self)
        self.input = input_q
        self.indices = indices_q
        self.result = result_q
        self.alat=alat
        self.alon=alon
        self.blat=blat
        self.blon=blon
        del alat,alon,blat,blon #clean memory
        self.na = len(self.alon)
        self.nb = len(self.blon)
    
    def task(self):
        
        #grabs host from queue
        input = self.input.get()
        index = input[1]
        id = input[0]
        
        if verbose > 0 : print "%s started at time: %s" % (self.getName(), datetime.datetime.now())

        #Map the first lon/lat pairs
        alat = self.alat[index]
        alon = self.alon[index]
        
        na = len(alat)
        nb = len(self.blat)
        
        dum = np.tile(np.deg2rad(alat), (nb, 1))
        dumdst = np.cos(dum)
        interm = np.sin(dum)
        
        del dum
        dum =  np.tile(np.deg2rad(self.blat).transpose(), (na, 1)).transpose()
        dumdst *= np.cos(dum)
        interm *= np.sin(dum)
        
        del dum
        dum = np.tile(np.deg2rad(alon), (nb, 1))
        dum -= np.tile(np.deg2rad(self.blon).transpose(), (na, 1)).transpose()
        dumdst *= np.cos(dum)
        
        del dum
        
        dumdst += interm
        dumdst=rt * np.arccos(dumdst)
        
        del interm
        
        self.indices.put(id)
        self.result.put(dumdst)
#        lock = threading.Lock()
#        lock.acquire()
##        dist_matrix[:,index] = dumdst
#        lock.release()
        del dumdst
#        return(dumdst)
        
        if verbose > 0 : print "%s ended at time: %s" % (self.getName(), datetime.datetime.now())
    
    def run(self):
        
            #Starts the queue
            self.task()
            #signals to queue job is done
            self.input.task_done()
       

def distance_matrix_parallel(alat,alon,blat,blon,N_thread=4):
    
    
    #Setup input and output queues
    input_q = Queue.Queue()
    indices_q = Queue.Queue()
    result_q = Queue.Queue()
    
    #Get dimensions to split matrix
    x=len(alat)
    y=len(blat)
    
    
    #Map the data along X axis
    step = np.ceil(np.float(x) / N_thread)
    if N_thread *step < x : raise '[ERROR] splitting of time series is not valid'
    
    #If the serie is not long enough, go back to sequential version
    if step <= N_thread :
        if verbose > 1 : print "[WARNING] series too short. back to sequential code"
        return distance_matrix(alat,alon,blat,blon)
    
    indices = []
    for i in np.arange(N_thread) : indices.append(np.arange(i*step,i*step+step))
    
    indices = []
    for i in np.arange(N_thread) :
        dumind = np.arange(i*step,i*step+step,dtype=np.long)
        dumind=dumind.compress((dumind >= 0) & (dumind <= x - 1))
        indices.append(dumind)
        
    
    #redefine output matrix
#    global dist_matrix
#    dist_matrix = np.zeros((x,y))    
    
    #spawn a pool of threads, and pass them queue instance 
    for i in range(N_thread):
        t = ThreadClass(input_q,indices_q,result_q,alat,alon,blat,blon)
        t.setDaemon(True)
        t.start()
    
    #Feed threads with data
    for i in range(N_thread):
        input_q.put((i,indices[i]))

    #wait on the queue until everything has been processed     
    input_q.join()
    
    #Sort threads
    tnb=[]
    for i in range(N_thread) : tnb.append(indices_q.get(i))
    tsort=np.argsort(tnb)
    
    #Get back the results for each thread in a list of results
    for i in np.arange(N_thread) :
        r=result_q.get(i)
        if i == 0 : dum = [r]
        else : dum.append(r)

    #Reorder data from each thread into output matrix
    for i in tsort :
        if i == tsort[0] : dist_matrix = dum[i]
        else : dist_matrix=np.append(dist_matrix,dum[i],1)
    
    if dist_matrix.shape != (y,x) :
        raise Exception('[ERROR]Output matrix is not coherent with input data - check matrix reconstruction')

    return(dist_matrix)

def distance_matrix_inline(lat_a, lon_a, lat_b, lon_b):
    
    na = np.size(lat_a)
    nb = np.size(lat_b)
    
    #Earth radius (km)
    rt = 6378.137
    
#    lon_a_tile = np.tile(np.deg2rad(lon_a), (nb, 1))
#    lat_a_tile = np.tile(np.deg2rad(lat_a), (nb, 1))
#    lon_b_tile = np.tile(np.deg2rad(lon_b).transpose(), (na, 1)).transpose()
#    lat_b_tile = np.tile(np.deg2rad(lat_b).transpose(), (na, 1)).transpose()

#    plt.figure()
#    plt.imshow(lon_a_tile,interpolation='NEAREST',aspect='auto')
#    plt.show()
#    
#    plt.figure()
#    plt.imshow(lon_b_tile,interpolation='NEAREST',aspect='auto')
#    plt.show()
    
#    interm = np.cos(lat_a_tile) * np.cos(lat_b_tile) * np.cos(lon_a_tile - lon_b_tile) + np.sin(lat_a_tile) * np.sin(lat_b_tile)
    
    #lower memory usage for huge matrices
    dum = np.tile(np.deg2rad(lat_a), (nb, 1))
    dist_matrix = np.cos(dum)
    interm = np.sin(dum)
    
    del dum
    dum =  np.tile(np.deg2rad(lat_b).transpose(), (na, 1)).transpose()
    dist_matrix *= np.cos(dum)
    interm *= np.sin(dum)
    
    del dum
    dum = np.tile(np.deg2rad(lon_a), (nb, 1))
    dum -= np.tile(np.deg2rad(lon_b).transpose(), (na, 1)).transpose()
    dist_matrix *= np.cos(dum)
    
    del dum
    
    dist_matrix += interm
    
    del interm
        
    dist_matrix = rt * np.arccos(dist_matrix)
    
    return dist_matrix

def distance_matrix(alat,alon,blat,blon,parallel=True,**kwargs):
    if parallel : return distance_matrix_inline(alat, alon, blat, blon,**kwargs)
    else : return distance_matrix_inline(alat, alon, blat, blon)


# -*- coding: utf-8 -*-
import numpy as np
import datetime
from dateutil import tz
from Tkinter import Tk
import inspect
import getpass, socket, platform #User and Host names
from warnings import warn
import collections
import __main__, os, sys

def get_zero_element(array):
    try : array = array.flat.next()
    except StopIteration : pass
    return array

def current_time():
    LOCAL=tz.tzlocal()
    GMT = tz.gettz('GMT')
    return datetime.datetime.now().replace(tzinfo=LOCAL).astimezone(GMT)

def bytscl(array, maximum=None , minimum=None , nan=0, top=255 ):
    """
    see http://star.pst.qub.ac.uk/idl/BYTSCL.html
    note that IDL uses slightly different formulae for bytscaling floats and ints. 
    here we apply only the FLOAT formula...
    """
    if maximum is None: maximum = np.nanmax(array)
    if minimum is None: minimum = np.nanmin(array)
    return np.maximum(np.minimum(((top+1.0)*(array-minimum)/(maximum-minimum)).astype(np.int16), top),0)

def scale_vector(x,bottom,top):
    return (x - bottom) / (top-bottom)

#Screen size widgets
def get_screen_size(units='p'):
    units='1'+units
    width = Tk().winfo_fpixels(str(Tk().winfo_screenwidth())+'p')/Tk().winfo_fpixels(units)
    height = Tk().winfo_fpixels(str(Tk().winfo_screenheight())+'p')/Tk().winfo_fpixels(units)
    return np.array([width,height])

def get_screen_dpi(units='1i'):
    return Tk().winfo_fpixels(units)

def where_list(list1,list2):
    index=[]
    for i in list1 :
        try:
            index.append(list2.index(i))
        except ValueError:
            index.append(-1)

    return index

def argresample(a,**kwargs):
    '''
    This function returns the positions needed for resampling a "gappy" vector 
    
    :keyword dt: Force sampling at a given resolution. Otherwise uses main data sampling (computed by getting the median value of the derivative)
    '''
    
    dt=kwargs.get('dt',np.median(deriv(a)))
    
    h,_=np.histogram(a, bins=(a.max()-a.min())/dt + 1, range=[a.min()-dt/2.,a.max()+dt/2.])
    
    return h.astype(bool)
    

def isiterable(item):
    """
    Check if item is iterable ans is not a string (or unicode)
    """
    if isinstance(item, collections.Iterable):
        if not isinstance(item,(str,unicode))  :return True
        else : return False
    else : False

##from scipy.interpolate import lagrange, BarycentricInterpolator, barycentric_interpolate
#def deprecated_deriv(*args):
#    if len(args) == 1 :
#        y = args[0]
#        x = np.arange(len(y))
#    if len(args) == 2 :
#        x = args[0]
#        y = args[1]
#    
#    dx = x[1:] - x[:-1]
#    dy = y[1:] - y[:-1]
#    
##    d2 = lagrange(np.arange(len(x)),np.arange(len(dx))+0.5,dy/dx)
#
#    dx = np.append(np.array(2*dx[1] - dx[0]),dx)
#    dx = np.append(dx,np.array(2*dx[-1] - dx[-2]))
#    
#    dy = np.append(np.array(2*dy[1] - dy[0]),dy)
#    dy = np.append(dy,np.array(2*dy[-1] - dy[-2]))
#    
##    dxint=lagrange(np.arange(nx),np.arange(nx-1)+0.5,dx)
##    dyint = lagrange(np.arange(nx),np.arange(nx-1)+0.5,dy)
#    
##    d = BarycentricInterpolator(dx)
##    dxint = lagrange(np.arange(len(dx)-1) + 0.5,np.arange(len(dx)),dx)
##    dyint = lagrange(np.arange(len(dx)-1) + 0.5,np.arange(len(dx)),dy)
#    d = lagrange(np.arange(len(x)) + 0.5,np.arange(len(dx)),dy/dx)
#    
##    p = barycentric_interpolate(np.arange(len(dx))+0.5,dy/dx,np.arange(len(dx)))
##    p(np.arange(len(dx)))
##    p(np.arange(len(dx)))
#    
#    return d


def deriv(*args):
    """
    ; Copyright (c) 1984-2009, ITT Visual Information Solutions. All
    ;       rights reserved. Unauthorized reproduction is prohibited.
    ;
    
    ;+
    ; NAME:
    ;    DERIV
    ;
    ; PURPOSE:
    ;    Perform numerical differentiation using 3-point, Lagrangian 
    ;    interpolation.
    ;
    ; CATEGORY:
    ;    Numerical analysis.
    ;
    ; CALLING SEQUENCE:
    ;    Dy = Deriv(Y)         ;Dy(i)/di, point spacing = 1.
    ;    Dy = Deriv(X, Y)    ;Dy/Dx, unequal point spacing.
    ;
    ; INPUTS:
    ;    Y:  Variable to be differentiated.
    ;    X:  Variable to differentiate with respect to.  If omitted, unit 
    ;        spacing for Y (i.e., X(i) = i) is assumed.
    ;
    ; OPTIONAL INPUT PARAMETERS:
    ;    As above.
    ;
    ; OUTPUTS:
    ;    Returns the derivative.
    ;
    ; COMMON BLOCKS:
    ;    None.
    ;
    ; SIDE EFFECTS:
    ;    None.
    ;
    ; RESTRICTIONS:
    ;    None.
    ;
    ; PROCEDURE:
    ;    See Hildebrand, Introduction to Numerical Analysis, Mc Graw
    ;    Hill, 1956.  Page 82.
    ;
    ; MODIFICATION HISTORY:
    ;    Written, DMS, Aug, 1984.
    ;    Corrected formula for points with unequal spacing.  DMS, Nov, 1999.
    ;-
    ;
    ; on_error,2              ;Return to caller if an error occurs
    """
    x = args[0]
    n = x.size
    if n < 3 : raise Exception('Parameters must have at least 3 points')


    if (len(args) == 2) :
        y=args[1]
        if n != y.size : raise 'Vectors must have same size'
    
        #;df/dx = y0*(2x-x1-x2)/(x01*x02)+y1*(2x-x0-x2)/(x10*x12)+y2*(2x-x0-x1)/(x20*x21)
        #; Where: x01 = x0-x1, x02 = x0-x2, x12 = x1-x2, etc.
    
        if isinstance(x,np.ma.masked_array) :  x = x.data                # Convert masked arrays to classic arrays
        if not isinstance(x,np.float) : x.astype(np.float) #;If not floating type, ensure floating... 
        
        x12 = x - np.roll(x,-1)                                      #;x1 - x2
        x01 = np.roll(x,1) - x                                       #;x0 - x1
        x02 = np.roll(x,1) - np.roll(x,-1)                           #;x0 - x2
    
        d = np.roll(y,1) * (x12 / (x01*x02)) \
            + y * (1./x12 - 1./x01) \
            - np.roll(y,-1) * (x01 / (x02 * x12))                    #Middle points
        
        
        #Formulae for the first and last points:
        d[0] = y[0] * (x01[1]+x02[1])/(x01[1]*x02[1]) \
            - y[1] * x02[1]/(x01[1]*x12[1]) \
            + y[2] * x01[1]/(x02[1]*x12[1])                          #;First point
        n2 = n-2
        d[n-1] = -y[n-3] * x12[n2]/(x01[n2]*x02[n2]) \
            + y[n-2] * x02[n2]/(x01[n2]*x12[n2]) \
            - y[n-1] * (x02[n2]+x12[n2]) / (x02[n2]*x12[n2])             #;Last point

    #Equally spaced point case
    else :
        d = (np.roll(x,-1) - np.roll(x,1))/2.
        d[0] = (-3.0*x[0] + 4.0*x[1] - x[2])/2.
        d[n-1] = (3.*x[n-1] - 4.*x[n-2] + x[n-3])/2.

    return d


def mask2NaN(array):
    n=array.size
    if array.mask.size != n :
        array.mask = np.zeros(n,dtype=bool)
#        raise np.ma.MaskError("[mask2NaN]Error : mask length is not consistent with data")
#        array.mask = np.ones(n,dtype=bool)
    array.data[np.arange(n).compress(array.mask)] = np.NaN
    return array


def histogram_indices(hist,R):
    ind = []
    for k in np.arange(len(hist)) : ind.append(R[R[k] : R[k+1]]) if hist[k] > 0 else ind.append([])
    return ind
#    for k in notempty : ind.append(R[R[k] : R[k+1]])



def nanargmin(array,axis=None):
    if axis is None : return np.nanargmin()

def nearest(t, x):
    adiff=np.abs(t-np.float(x))
    i=np.argmin(adiff)
    return i

def cart2polar(x, y):
    r = np.sqrt(x**2 + y**2)
    theta = np.arctan2(y, x)
    return r, theta

def polar2cart(r, theta):
    x = r * np.cos(theta)
    y = r * np.sin(theta)
    return x, y

def rad2geo(alpha):
    theta=(np.rad2deg(alpha)*-1)+360+90
    theta=np.mod(theta,360)
    return theta

def cart2geo(u,v):
    spd,d=cart2polar(u, v)
    d=rad2geo(d)
    return spd,d

def rms(array):
    """
    ;+
    ; RMS : Returns root-mean-squared deviation of an array
    ; 
    ; @param array {in}{required}{type:NUMERIC} 2-dimensionnal array. Time dimension<br />
    ;   should be the last dimension.
    ;   
    ; @returns RMS array (1 value per time serie)
    ; 
    ; @author Renaud DUSSURGET, LEGOS/CTOH
    ;-
    """
    #;time should be the last dimension
      
#    IF sz[ndims-1] EQ 1 THEN RETURN, 0.
    
    nans = np.where(~np.isnan(array))
    cnt=len(nans)
    
    if (cnt == 0) : return np.NaN
    
    nval=np.nansum(~np.isnan(array))
    
    mn = np.nansum(array) / nval

    return np.sqrt(np.nansum((array - mn)**2.0)/nval)
#    RETURN, SQRT((TOTAL((array - REBIN(mn,sz))^2D,ndims,/DOUBLE,/NAN) / nval))
#;  RETURN, SQRT((TOTAL((array - REBIN(mn,sz))^2D,ndims,/DOUBLE,/NAN) / sz[ndims -1]))

def get_caller(level=2):
        frame=inspect.currentframe()
        for l in np.arange(level) : frame=frame.f_back
        code=frame.f_code
        return code

def get_main():
#     return os.path.basename(__main__.__file__)
    return ' '.join(sys.argv[:])

def username():
    return getpass.getuser()

def hostname(full=False):
    if full:
        return socket.gethostbyaddr(socket.gethostname())[0]
    else : return platform.node()

def message(MSG_LEVEL,msg,verbose=1):
    """
     MESSAGE : print function wrapper. Print a message depending on the verbose level
     
     @param MSG_LEVEL {in}{required}{type=int} level of the message to be compared with self.verbose
     
     @example self.message(0,'This message will be shown for any verbose level') 
    """
    
    caller=get_caller()
    if MSG_LEVEL <= verbose :  print('[{0}.{1}()] {2}'.format(__name__,caller.co_name,msg))

def warning(MSG_LEVEL,msg,verbose=1):
    """
     WARNING : Wrapper to the warning function. Returns a warning when verbose level is not 0. 
     
     @param MSG_LEVEL {in}{required}{type=int} level of the message to be compared with self.verbose
     
     @example self.waring(1,'Warning being issued) 
    """
    
    if verbose <= 1 : warn(msg)



#!/home/rdussurget/.virtualenvs/compile.octant.UBU64/bin/python
# encoding: utf-8
'''
altimetry.tools.cdf_merge -- shortdesc

altimetry.tools.cdf_merge is a description

It defines classes_and_methods

@author:     user_name
            
@copyright:  2013 organization_name. All rights reserved.
            
@contact:    user_renaud.dussurget@gmail.com
'''

import sys
import os

from optparse import OptionParser
from altimetry.tools.nctools import nc

__all__ = []
__version__ = 0.1
__date__ = '2013-06-18'
__updated__ = '2013-06-18'

DEBUG = 0
TESTRUN = 0
PROFILE = 0

def main(argv=None):
    '''Command line options.'''
    
    program_name = os.path.basename(sys.argv[0])
    program_version = "v0.1"
    program_build_date = "%s" % __updated__
 
    program_version_string = '%%prog %s (%s)' % (program_version, program_build_date)
    #program_usage = '''usage: spam two eggs''' # optional - will be autogenerated by optparse
    program_longdesc = '''''' # optional - give further explanation about what the program does
    program_license = "Copyright 2013 Renaud Dussurget"
 
    if argv is None:
        argv = sys.argv[1:]
    try:
        # setup option parser
        parser = OptionParser(version=program_version_string, epilog=program_longdesc, description=program_license)
        parser.add_option("-v", "--verbose", dest="verbose", action="store", type="int", help="set verbosity level [default: %default]",default=0)
        parser.add_option("-l", "--use_local_dims", dest="use_local_dims", action="store_true", help="Use local netcdf dimensions [default: %default]",default=True)
        parser.add_option("-O", "--overwrite", dest="overwrite", action="store_true", help="Overwrite output file [default: %default]",default=False)
        parser.add_option("-f", "--format", dest="format", action="store", help="Output NetCDF format [default: %default]",default="NETCDF4")

        
        #Add kwargs!
        kwargs_keys=[]
        for a in argv :
            if a.startswith('--'):
                parser.add_option(a,dest=a.replace('--',''),action="store", help="Additional keyword to use with nctools [default: %default]")
#                parser.set_defaults(**{a.replace('--',''):False})
                kwargs_keys.append(a.replace('--',''))

        # set defaults
#        parser.set_defaults(verbose="4",use_local_dims="True")
        
        # process options
        (opts, args) = parser.parse_args(argv)
        
        if opts.verbose > 0:
            print("verbosity level = %d" % opts.verbose)
            
        if len(args) != 3 :
            raise Exception('Program needs 3 arguments') 
            
        # MAIN BODY #
        
    except Exception, e:
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2

    #Get program arguments
    f1=args[0]
    f2=args[1]
    fout=args[2]
    
    #Get keyword arguments for nc
    kwargs={}
    for k in kwargs_keys :
        kwargs.update({k:eval(opts.__dict__.get(k))})
    
    ncr=nc(verbose=opts.verbose,use_local_dims=opts.use_local_dims)
    
    d1=ncr.load(f1,**kwargs)
    d2=ncr.load(f2,**kwargs)
    
    dout=d1.copy()
    for k in d2.keys():
        dout.update({k:d2.get(k)})
    
    attrStr=dout.get('_attributes',{})
    
    cmd=[os.path.basename(sys.argv[0])]
    for a in argv : cmd.append(a)
    attrStr.update({'history':attrStr.pop('history','') +' '.join(cmd)})
    dout.update({'_attributes':attrStr})
    ncr.write(dout,fout,clobber=opts.overwrite,format=opts.format)


if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-h")
    if TESTRUN:
        import doctest
        doctest.testmod()
    if PROFILE:
        import cProfile
        import pstats
        profile_filename = 'altimetry.tools.cdf_merge_profile.txt'
        cProfile.run('main()', profile_filename)
        statsfile = open("profile_stats.txt", "wb")
        p = pstats.Stats(profile_filename, stream=statsfile)
        stats = p.strip_dirs().sort_stats('cumulative')
        stats.print_stats()
        statsfile.close()
        sys.exit(0)
    
    sys.exit(main())



# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.pylab as pylab

#from grid.regridding import griddata, xy2xy

from netCDF4 import Dataset as ncfile
import glob
import os
from altimetry.tools import recale, in_limits, where_list, recale_limits, get_caller, get_main, isiterable, username, hostname, current_time
#import altimetry.data.alti_tools as atools
from collections import OrderedDict
from warnings import warn
from copy import deepcopy

class baseDict(OrderedDict):
    
    def __init__(self, *args, **kwargs):
        keys=kwargs.pop('keys',None)
        values=kwargs.pop('values',None)
         
        if len(args) == 0 : args =(self,)
        
        #If 2 arguments, both list arguments become keys and args 
        elif len(args) == 2 :
            if (isinstance(args[0],list) or isinstance(args[0],tuple)) and (isinstance(args[1],list) or isinstance(args[1],tuple)) and (keys is None) and (values is None) :
                keys,values=args
                args = (self,)
        #else len(args) == 1 : an object instance is passed
        
        super(baseDict,self).__init__(*args,**kwargs)
        
        if keys is not None and values is not None :
            self.add(keys,values)
        
    def copy(self):   
        return deepcopy(self)

class dimStr(baseDict):
    '''
    A dimension structure based on OrderedDict.
    '''
    
    def __init__(self,*args,**kwargs):
        super(dimStr, self).__init__(*args, **kwargs)
        self.set_ndims(__init__=True)
        
    def get_ndims(self):
        return len([k for k in self.keys() if k is not '_ndims'])
    
    def set_ndims(self,__init__=False,__setitem__=False):
        #Modify _ndims if it exists or during __init__
        if self.has_key('_ndims') or __init__ :
            
            self['_ndims']=self.get_ndims()
            
            #Force pushing _ndims in first
            if (not __setitem__) : self.reorder()
            
    
    def reorder(self):
        if self.has_key('_ndims') :
            if self['_ndims'] > 0 :
                for k in self.keys():
                    self[k]=self.pop(k)
                pass
            
    
    #===========================================================================
    # Work with dimensions
    #===========================================================================
    def add(self,dimlist,dimvalues):
        '''
        add dimensions
        
        :parameter dimlist: list of dimensions
        :parameter dimvalues: list of values for dimlist
        '''
        for i,d in enumerate(dimlist):
            self[d] = dimvalues[i]
        
        self.set_ndims()
    
    def put(self,*args):
        '''
        Wrapper to add
        '''
        self.add(*args)
        
        
    def get(self,dimlist):
        '''
        get dimensions
        :parameter dimlist: list of dimensions
        '''
        out=()
        for i,d in enumerate(dimlist):
            out+=(super(dimStr, self).get(d,None),)
         
        return out
    
    def pop(self,*args):
        '''
        pop dimensions
        :parameter dimlist: list of dimensions
        :return: popped dimensions (tuple)
        
        .. note: overrides :func:`OrderedDict.pop`
        '''
        nargs=len(args)
        dimlist=args[0] if isiterable(args[0]) else [args[0]]
        missing=args[1] if nargs > 1 else None 
        out=()
        for d in dimlist:
            out+=(super(dimStr, self).pop(d,missing),)
            if out[-1] == missing : self.set_ndims()
        
        return out if len(out) > 1 else out[0]

    def __setitem__(self, *args,**kwargs):
        OrderedDict.__setitem__(self, *args, **kwargs)
        if args[0] != '_ndims' :self.set_ndims(__setitem__=True)

    def keys(self):
        return [k for k in super(dimStr, self).keys() if k != '_ndims']

    def values(self):
        return [self[k] for k in super(dimStr, self).keys() if k != '_ndims']

class attrStr(baseDict):
    '''
    An attribute structure based on OrderedDict.
    '''
    
#     def __init__(self,*):
#         super(attrStr,self).__init__()
#         if attrlist is not None : self.add(attrlist,attrlist)
    
    def add(self,attrlist,attrvalues):
        '''
        add an attribute
        
        :parameter dimlist: list of dimensions
        :parameter dimvalues: list of values for dimlist
        '''
        for i,d in enumerate(attrlist):
            self[d] = attrvalues[i]
    
    def put(self,*args):
        '''
        Wrapper to addDim
        '''
        self.add(*args)
        
        
#     def get(self,dimlist):
#         '''
#         '''
#         out=()
#         for d in enumerate(dimlist):
#             out+=(self['_dimensions'].get(d,None),)
#         
#         return out
    
    def pop(self,*args):
        '''
        pop dimensions from data structure.
        :parameter dimlist: list of dimensions
        :return: popped dimensions (tuple)
        
        .. note: overrides :func:`OrderedDict.pop`
        '''
        
        nargs=len(args)
        dimlist=args[0] if isiterable(args[0]) else [args[0]]
        missing=args[1] if nargs > 1 else None
        out=()
        for d in dimlist:
            out+=(super(attrStr, self).pop(d,missing),)
        
        return out if len(out) > 1 else out[0]
    
class dataStr(baseDict):
    '''
    Data structure class to be used with altimetry.tools.nctools.nc
    '''
    
    def __init__(self,*args,**kwargs):
        super(dataStr,self).__init__(*args,**kwargs)
        self['_dimensions']=dimStr()
        '''
        Dimensions structure
        '''
        
        self['_attributes']=attrStr()
        '''
        Attributes structure
        '''
    
    def add(self,struct):
        if isinstance(struct,varStr): self[struct.getVarName(self)]=struct
        elif isinstance(struct,attrStr): self['_attributes'].put(struct)
        elif isinstance(struct,dimStr): self['_dimensions'].put(struct)
    
    #~ def append(self,struct,dimension=None):
	#~ 
	#~ if dimension is None :
	    #~ dimension=self[self.getStructDims()]=stuct.keys()[0]
	#~ 
	#~ #Append data
	#~ for k in self.getStructVars():
	    #~ 
	    #~ np.ma.concatenate()
	    #~ self[struct.getVarName(self)]['data']+=
	#~ 
	#~ #Append dimensions
	#~ for k in self.getStructVars():
    
    def put(self,*args):
        self.add(*args)
    
    def getStructVars(self):
        return [k for k in self.keys() if isinstance(self[k],varStr)]
    
    def getStructDims(self):
        return [k for k in self.keys() if isinstance(self[k],dimStr)]
    
    def getStructAttr(self):
        return [k for k in self.keys() if isinstance(self[k],attrStr)]
    
class varStr(dataStr):
    '''
    Data structure class containing variables
    
    :keyword name: Name of the variable. Stored in :attr:`varStr.name`
    :keyword (dimStr,dict or OrderedDict) dimensions: Dimension structure .
    :keyword dimlist: list of variable dimension names. Their order will be kept.
    :keyword optional dimvalues: list of values associated with dimlist.
        
    :keyword (attrStr,dict or OrderedDict) attributes: Attribute structure.
    :keyword attrlist: list of variable attribute names. Their order will be kept.
    :keyword optional attrvalues: list of values associated with attrist.
    
    
    .. note : if dict are used in :param:`dimensions` and :param:`attributes`, dimensions may not be in proper order. Use dimlist and attrlist instead to keep order.
      
    '''
    
    def __init__(self,
                 name=None,
                 dimensions=None,
                 dimlist=None,
                 dimvalue=None,
                 attributes=None,
                 attrlist=None,
                 attrvalues=None,
                 data=None):
        
        #Init parent object
        dataStr.__init__(self)
        
        #Set variable name
        self.name = name
        
        #Set variable dimensions
        if dimensions is not None and isinstance(dimensions,(dimStr,dict,OrderedDict)):
            dimlist = dimensions.keys()
            dimvalue = dimensions.values()
        if dimlist is not None :
            if dimvalue is None : dimvalue = [None for d in dimlist]
            self['_dimensions'].add(dimlist,dimvalue)
        
        #Set variable attributes
        if attributes is not None and isinstance(attributes,(attrStr,dict,OrderedDict)):
            attrlist = attributes.keys()
            attrvalues = attributes.values()
        if attrlist is not None :
            self['_attributes'].add(attrlist,attrvalues)
        
        #Set variable data
        if data is not None : self['data']=data
        
        #Reform attributes
        for d in self['_attributes'].keys():
            self[d] = self['_attributes'].pop(d)
        self.pop('_attributes')
        
    def getVarName(self,parent):
        
        name = None
        if hasattr(self, 'name') : name =  self.__dict__.pop('name',None)
        
        if name is None :
            varNames=parent.getStructVars()
            if len(np.shape(self['data'])) == 1 :
                nvars = len([v for v in varNames if v.startswith("Param_")])
                name =  "Param_%04i" % (nvars+1)
            elif len(np.shape(self['data'])) == 2 :
                nvars = len([v for v in varNames if v.startswith("Grid_")])
                name = "Grid_%04i" % (nvars+1)
            elif len(np.shape(self['data'])) == 3 :
                nvars = len([v for v in varNames if v.startswith("Matrix_")])
                name = "Matrix_%04i" % (nvars+1)
        
        return name

class ncStr(dataStr):
    '''
    Data structure class containing NetCDF data.
    '''
    
    def __init__(self,
                 dimlist=None,
                 dimvalues=None,
                 dimensions=None,
                 attrlist=None,
                 attrvalues=None,
                 attributes=None,
                 **kwargs):
        
        dataStr.__init__(self)
        if dimlist is not None : self['_dimensions'].add(dimlist,dimvalues)
        elif dimensions is not None : self['_dimensions']=dimensions
        else : self['_dimensions']=dimStr()
        
        if attrlist is not None : self['_attributes'].add(attrlist,attrvalues)
        elif attributes is not None : self['_attributes'].add(attributes)
        self.set_global_attributes(**kwargs)
#         self['_attributes'].update(set_global_attributes(**kwargs))

    def set_global_attributes(self,full=False):
        
        #Set user
        user="%s@%s" % (username(),hostname(full=full))
        if not self['_attributes'].has_key('user'): self['_attributes'].update({'user':user})
        
        #Set history
        history=self['_attributes']['history'].split('\n') if self['_attributes'].has_key('history') else [] 
        history+=["%s : %s" %(current_time(), get_main())]
        self['_attributes'].update({'history':'\n'.join(history)})

class nc :
    
    '''
    A class for easy-handling of NetCDF data based on :mod:`NetCDF4` package.
    
    :example: To load different sets of data, try these :
    
      * Simply load a NetCDF file
         
        * The file has standard dimensions (eg. called longitude & latitude)
        
        .. code-block:: python
         
           ncr=nc()
           data=ncr.read(file)
         
           lon=data.lon
           lat=data.lat
           Z=data.Z
        
        * We do not want to match for standard dimension names and keep original names
        
        .. code-block:: python
           :emphasize-lines: 2,4-5
         
           ncr=nc()
           data=ncr.read(file,use_local_dims=True)
           
           lon=data.longitude
           lat=data.latitude
           Z=data.Z
        
         
        * We extract a region and depth range between 2 dates:
        
          * We extract between 30-40°N & 15-20°E (limit).
          * We extract between 100 & 200 m deep (depth).
          * We get data from 2010/01/01 to 2010/01/07 (time).
          * File has standard dimensions called longitude, latitude, level and time
        
        .. code-block:: python
           :emphasize-lines: 7-9
         
           ncr=nc()
           limit=[30,15,40,20]
           depth=[100,200]
           time=[21915,21921]
           
           data=ncr.read(file,
                         limit=limit,
                         timerange=time,
                         depthrange=depth)
             
           lon=data.lon
           lat=data.lat
           dep=data.depth
           dat=data.time
           Z=data.Z
    
    
      * More sophisticated example using a file containing bathymetetry data
        
        * Load a file and extract a regions and subsample it to a lower resolution
         
          * The file has dimensions NbLongitudes & NbLatitudes.
          * We extract between 30-40°N & 15-20°E (limit).
          * We subsample every 3 points (stride).
       
        .. code-block:: python
           :emphasize-lines: 5-6
        
           limit=[30,15,40,20]
           stride = (3,)
           ncr=nc(use_local_dims=True)
           bathy=ncr.load(file,
                          NbLongitudes=(limit[1],limit[3])+stride,
                          NbLatitudes=(limit[0],limit[2])+stride)
       
        
        * Then we save the data to another file (output).
       
        .. code-block:: python
          
           #save data
           bathy.write_nc(output)
        
        * We update the **history** global attribute of data structure
       
        .. code-block:: python
           :emphasize-lines: 9,12
          
           #Get attribute structure
           attrStr=bathy.get('_attributes',{})
            
           #Get arguments called from the shell
           cmd=[os.path.basename(sys.argv[0])]
           for a in argv : cmd.append(a)
            
           #update attribute stucture (pop history and concatenate with current commands=.
           attrStr.update({'history':attrStr.pop('history','')+' '.join(cmd)+'\\n'})
            
           #update NetCDF data structure
           bathy.update({'_attributes':attrStr})
          
           #save data
           bathy.write_nc(output)
        
        
        * We now want to flag all values from variable Z above 0 by setting them to fill_value and append this modified variable to the output file
      
        .. code-block:: python
          
           #load variable
           Z = bathy.Z
         
           #flag variable
           Z.mask[Z >= 0] = False
         
           #update attributes
           Z['_attributes']['long_name'] = 'flagged bathymetry'
         
           #append modified bathymetry to a variable named Z_2 in output file.
           bathy.push(output,'Z2',Z)
           
    '''
    
    def __init__(self, limit=[-90., 0., 90., 360.], verbose=0, zero_2pi=False, transpose=False, use_local_dims=False, **kwargs):
        '''
        Returns an :class:`altimetry.tools.nctools.nc` instance.
        
        :keyword limit: the limits of the domain to handle ([latmin,lonmin,latmax,lonmax]).
        :keyword verbose: verbosity level on a scale of 0 (silent) to 4 (max verobsity)
        :keyword zero_2pi: limits goes from 0 to 360 degrees (not -180/180).
        :keyword use_local_dims: use file dimensions instead of trying to detect standard dimension names (cf. :attr:`altimetry.tools.nctools.nc.use_local_dims`)
        
        '''
        
        #Init system variables
        self.zero_2pi = zero_2pi
        
        self.limit = np.array(recale_limits(limit, zero_2pi=self.zero_2pi))
        '''
        limits of the domain : [latmin,lonmin,latmax,lonmax] (default = [-90.,0.,90.,360.])
        
        .. note:: limits are automatically reset using :func:`altimetry.tools.recale_limits`
        '''
        
        self.verbose = verbose
        '''
        verbosity level on a scale of 0 (silent) to 4 (max verbosity)
        '''
        
        self.fileid = np.array([])
        '''
        array of file IDs
        '''
        
        self.count = 0
        '''
        number of files loaded
        '''
        
        self.size = 0
        '''
        length of the dataset
        '''

        self.use_local_dims=use_local_dims
        '''
        this option prevent from trying to detect standard CF dimensions such longitude, latitude, time in the file and keep the original dimensions of the file
        
        .. note:: Set this option to True when file is not standard (eg. not following CF conventions).
        
        .. note:: Normal behaviour is to match dimensions (ie. a dimension and the associated variable of the same name) with specific names. Resulting variables associated with these dimensions will be called :
          * lon (longitudes) : matches dimensions starting with 'lon'
          * lat (latitudes) : matches dimensions starting with 'lat'
          * time (time) : matches dimensions starting with 'date' or 'time'
          * depth (date) : matches dimensions starting with 'dep' or 'lev'
          
        '''
    
    def push(self,*args,**kwargs):#file,varName,value,**kwargs):
        '''
        append a variable from a given data structure to the existing dataset.
        
        :parameter optional file:
        :parameter name: variable name
        :parameter value: data
        
        :keyword start: broadcast the data to a portion of the dataset. starting index.
        :keyword counts: broadcast the data to a portion of the dataset. number of counts.
        :keyword stride: broadcast the data to a portion of the dataset. stepping along dimension.
        
        '''
        
        reload = False
        
        #Get arguments
        largs=list(args)
        if os.path.exists(largs[0]) :
            file = largs.pop(0)
            self._filename = file
            reload = True
        name = largs.pop(0)
        value = largs.pop(0)
        
        #Get keywords
        start = kwargs.get('start',None)
        counts = kwargs.get('counts',None)
        stride = kwargs.get('stride',None)
        
        
        file = self._filename
        
        if self.verbose == 1 : self.message(1, 'Writing data into file {0}'.format(os.path.basename(file)))
        elif self.verbose > 1 : self.message(2, 'Writing data into file {0}'.format(file))
        ncf=ncfile(file, 'a', format='NETCDF4', clobber=False)
     
        #Get list of recorded parameters:
        par_list = ncf.variables.keys()
        
        #Check if variable exists
        if 'sla' not in par_list : createVariable = True
        else : createVariable = False
        
        
        if not createVariable :
            var=ncf.variables[name]
            
            #Get dimensions
            fileDims=OrderedDict({'_ndims':len(var.dimensions)})
            [fileDims.update({str(d):len(ncf.dimensions[d])}) for d in var.dimensions]
            
            if start is None : start = [0 for sh in var.shape]
            else : start = start[::-1]
            if counts is None : counts = [sh for sh in var.shape]
            else : counts = counts[::-1]
            if stride is None : stride = [1 for sh in var.shape]
            else : stride = stride[::-1]
            
            #Transpose before write
            value=np.transpose(value)
            strides=var[:].strides
            
            #Construct indices (cf.http://docs.scipy.org/doc/numpy-1.5.x/reference/arrays.indexing.html)
            ind=()
            for i in np.arange(len(start)) : ind+=(slice(start[i],start[i]+counts[i],strides[i]),)
            
            #Push data into file
            try : var[ind]=value[:]
            except ValueError : self.Error('Input variable {0} {1} cannot be broadcasted into {2}'.format(name,value.shape,os.path.basename(file)))
            
            #Check if variable is in agreement with dimensions
#            dimStr=self._dimensions #Get dimensions  
            
            ncf.close()
            
            
        else :
            self.Error('This option is not yet implemented')
        
        
        
    
    def write(self, data, outfile, clobber=False,format='NETCDF4'):
        '''
        Write a netCDF file using a data structure.
        
        :parameter data: data structure
        :parameter outfile: output file
        :keyword clobber: erase file if it already exists
        :keyword format: NetCDF file format.
        
        .. note :: the data structure requires a "_dimensions" field (dimension structure)
        '''
        
        #Open file
        if self.verbose == 1 : self.message(1, 'Writing data file {}'.format(os.path.basename(outfile)))
        elif self.verbose > 1 : self.message(2, 'Writing data file {}'.format(outfile))
        root_grp=ncfile(outfile, 'w', format=format, clobber=clobber)
#        root_grp.description = 'nctools.write() file'
        
        #Get attributes
        if data.has_key('_attributes'):
            self.message(2, 'Adding attributes data')
            attrStr=data.pop('_attributes')
            self.message(4, 'Attribute list (N={0}) :[{1}]'.format(len(attrStr.keys()), ','.join(attrStr.keys())) )
            root_grp.setncatts(attrStr)
        
        #Get dimensions
        dimDict=data.pop('_dimensions')
        ndims=dimDict.pop('_ndims')
        dimlist = dimDict.keys()
        dimVal = dimDict.values()
        
        #Get variables
        parlist = data.keys()
        
        
        # Set up dimensions
        
        #Put dimensions
        for d in dimlist :
            self.message(2, 'Adding D {0}={1}'.format(d,dimDict[d]))
            root_grp.createDimension(d, dimDict[d])
        
#        if data.has_key('p')  : self.Error('Variable name \'p\' is not available for use as NetCDF variable name')
        
        #Loop over variables
        for p in parlist :
            
            self.message(4,'Looping parameter list : %s' %p)
            
            #Get dimensions for current variable
            if not data[p].has_key('_dimensions') : warn('_dimension attribute is not set for variable'+p)
            pardim=data[p].pop('_dimensions') if data[p].has_key('_dimensions') else dimStr()
            if isinstance(pardim,dimStr) : pardim=tuple(pardim.keys())
            elif isinstance(pardim,dict) :pardim=tuple(pardim.keys()[1:]) if pardim.has_key("_ndims") else tuple(pardim.keys())
            elif isinstance(pardim,list) : pardim = tuple(pardim)
            elif isinstance(pardim,tuple) : pass
            else : self.Error('_dimensions must be dict, list or tuple - not {0}'.type(pardim))
            
            #Convert to numpy array if scalar or non numpy
            if not hasattr(data[p]['data'],'__iter__') or not hasattr(data[p]['data'], 'dtype'):  data[p]['data']=np.array(data[p]['data'])

            #Get _FillValue if possible
#            if hasattr(data[p]['data'],'fill_value') : data[p]['_FillValue']=data[p]['data'].fill_value 

#            if not (data[p]['data'].dtype == '|S6') and not (data[p]['data'].dtype == '|S2') :
            self.message(2, 'Adding V {0} (dims={{{1}}},attr={{{2}}})'.
                         format(p,
                                ', '.join(['\'{0}\':{1}'.format(d,dimDict[d]) for d in pardim]),
                                ', '.join(['\'{0}\':{1}'.format(d,data[p][d]) for d in data[p].keys() if (d != '_dimensions') and (d != 'data')]) )
                         )
            
            if locals().has_key(p) : self.Error('Variable name \'{0}\' is already declared - change variable name'.format(p))
            
            #Scale it back
            ##############
            
            #look at scaler factors and offsets in both attributes and data structure keys
            scale_factor=scale_factor=data[p].get('scale_factor',None)
            if scale_factor is None : scale_factor=data[p].get('scale',None)
            if scale_factor is None and hasattr(data[p]['data'],'__dict__') : scale_factor=data[p]['data'].__dict__.get('scale_factor',None)  
            if scale_factor is None and hasattr(data[p]['data'],'__dict__') : scale_factor=data[p]['data'].__dict__.get('scale',None) 
            
            
            offset=data[p].get('add_offset',None) 
            if offset is None and hasattr(data[p]['data'],'__dict__') : offset=data[p]['data'].__dict__.get('add_offset',None)
            
            #Apply scaling and prepare message
            scale_msg='Apply scaling : {}'.format(p)
            if scale_factor is not None :
                data[p]['data'] = data[p]['data'] / scale_factor
                scale_msg+=' / {0}'.format(scale_factor)
            if offset is not None :
                data[p]['data'] = data[p]['data'] - offset
                scale_msg+=' - {0}'.format(offset)
            
            #show message
            if scale_factor is not None or offset is not None :
                self.message(2,scale_msg)
            
            #Apply fill_value as argument to createVariable function
            if hasattr(data[p]['data'],'fill_value') :
                locals()[p] = root_grp.createVariable(p,
                                               data[p]['data'].dtype if not str(data[p]['data'].dtype).startswith('|S') else 'S1',
                                               pardim,
                                               fill_value=data[p]['data'].fill_value)
            else :
                locals()[p] = root_grp.createVariable(p,
                                               data[p]['data'].dtype if not str(data[p]['data'].dtype).startswith('|S') else 'S1',
                                               pardim)
            
            #Get data and compare to current dimensions
            dumVar=data[p].pop('data')
            ncsh=tuple([long(dimVal[dimlist.index(d)]) for d in pardim if d in dimlist])
            ncdimname=tuple([d for d in pardim if d in dimlist])
            
            #Transposition is done if provided dimensions is not in phase with data dimensions (and if provided dimensions has no unlimited dimensions)
            if not ncsh == dumVar.shape and ncsh.count(0) == 0:
                dimOrder=tuple([ncsh.index(dsh) for dsh in dumVar.shape if dsh in ncsh])
                self.message(2, 'Transposing data axes {0}{1}'.format(ncdimname,dimOrder)) #Make it a bit more explicit...
                if len(dimOrder) > 0:
                    dumVar=dumVar.transpose(dimOrder)
            
            #Transpose data before writing it into file
            locals()[p][:]=dumVar
            #.transpose(tuple(range(len(pardim))[::-1])) #old stuff
                
            #Update with attribute list
            attrDict=locals()[p].__dict__
            attrDict.update(data[p])
            [attrDict.pop(k) for k in  locals()[p].__dict__.keys()]
            
            
            #Get common attributes first
            attrDict.pop('_FillValue',None)
            locals()[p].setncatts(attrDict)
#            for a in attrDict :
#                if not hasattr(locals()[p],a) :
#                    locals()[p].setncattr(a,data[p][a])
        
        self.message(2, 'Closing file')
        root_grp.close()

        return True
        
        
    
    def read(self,file_pattern,**kwargs):
        '''
        Read data from a NetCDF file
        
        :parameter file_pattern: a file pattern to be globbed (:func:`glob.glob`) or a file list.
        :keyword kwargs: additional keywords to be passed to :meth:`altimetry.tools.nctools.nc.load` (eg. extracting a subset of the file)
        ''' 
        
        #Setup file list
        if isinstance(file_pattern, str) : ls = glob.glob(file_pattern)
        elif isinstance(file_pattern,np.ndarray) :
            ls = file_pattern.tolist()
            file_pattern = file_pattern[0]
        
        if len(ls) == 0 :
            self.Error('File pattern not matched : ' + file_pattern)
                
        self.filelist = [os.path.basename(i) for i in ls]
        self.filelist_count = [0] * len(self.filelist)
        enum = list(enumerate(ls))
        enum = zip(*enum)
        
        self.fid_list = np.array(enum[0])
        self.dirname = os.path.dirname(os.path.abspath(file_pattern))
        
        self.par_list = np.array([])
        self.dim_list = np.array([])
        self._dimensions = OrderedDict({'_ndims':0})
         
        #Loop over data files
        #####################
        for i in np.arange(len(self.fid_list)) :
            
            #Read data file
            ###############
            filename = os.path.join(self.dirname,self.filelist[i])
            self.message(1, "Loading " + os.path.basename(filename))
            
            res = self.load(filename, **kwargs) #read() function is specific of each class
            #self.update_dataset(res) #update class with loaded data
            
            #self.check_variables()
        
        #self.update()

        return(res)

    def load(self, filename, params=None, force=False, depthrange=None, timerange=None, output_is_dict=True, **kwargs):
        """
        NetCDF data loader
        
        :parameter filename: file name
        :parameter params: a list of variables to load (default : load ALL variables).
        
        :parameter depthrange: if a depth dimension is found, subset along this dimension.
        :parameter timerange: if a time dimension is found, subset along this dimension. 

        .. note:: using :attr:`altimetry.tools.nctools.limit` allows subsetting to a given region.

        :parameter kwargs: additional arguments for subsetting along given dimensions.
        
        .. note:: You can index along any dimension by providing the name of the dimensions to subsample along. Values associated to the provided keywords should be a length 2 or 3 tuple (min,max,<step>) (cf. :func:`altimetry.data.nctools.load_ncVar`).
        
        :keyword output_is_dict: data structures are dictionnaries (eg. my_hydro_data.variable['data']). If false uses an object with attributes (eg. my_hydro_data.variable.data). 
        
        :return  {type:dict} outStr: Output data structure containing all recorded parameters as specificied by NetCDF file PARAMETER list.
        
        :author: Renaud Dussurget
        

        """
        
        if (params is not None) & isinstance(params,str): params=[params]
        
        #Open file
        self._filename = filename
        try:
            ncf = ncfile(self._filename, "r")
        except Exception,e:
            warn(repr(e),stacklevel=2)
            return {}
        
        #Load global attributes
        akeys = ncf.ncattrs()
        attrStr=OrderedDict()
        for A in akeys : attrStr.update({A:ncf.getncattr(A)})
     
        #Get list of recorded parameters:
        dum = ncf.variables.keys()
        nparam = np.shape(dum)[0]
    #        par_list=np.array([''.join(ncf.variables.keys()[0,i,:].compressed()) for i in np.arange(nparam)])
        par_list = np.array(['{0}'.format(v) for v in ncf.variables.keys()])
        
        #remove empty items and update nparam
        par_list = par_list.compress([len(par) != 0 for par in par_list])
        nparam = par_list.size
        
        if nparam == 0 : self.Error('File has no data ({0})'.format(self._filename))
        
        #Get dimensions
        ncdimlist = np.array(['{0}'.format(d) for d in ncf.dimensions.keys()])
        ndims = len(ncdimlist)
        dimStr = OrderedDict()
        dimStr.update({'_ndims':ndims})
    
        if ndims == 0 : self.Error('File has no dimensions ({0})'.format(self._filename))
        
        #Check for the presence of strategic dimensions
        checkedDims = np.array(['lon', 'lat', 'time', 'depth'])
        existDim = -np.ones(4,dtype=int)
        if not self.use_local_dims :
            for i,d in enumerate(ncdimlist) :
                if ( (d.lower().startswith('lon')) | (d.lower().find('longitude') != -1) ) & (d.find('LatLon') ==-1) : existDim[0]=i
                if ( (d.lower().startswith('lat')) | (d.lower().find('latitude') != -1) ) & (d.find('LatLon') ==-1): existDim[1]=i
                if (d.lower().startswith('time')) | (d.lower().startswith('date')) : existDim[2]=i
                if (d.lower().startswith('lev')) | (d.lower().startswith('dep')) : existDim[3]=i
                    
#            existDim[0] = np.where([d.lower().startswith('lon') | (d.lower().find('longitude') != -1) for d in ncdimlist])[0]
#            existDim[1] = np.where([d.lower().startswith('lat') | (d.lower().find('latitude') != -1) for d in ncdimlist])[0]
#            existDim[2] = np.where([(d.lower().startswith('time')) | (d.lower().startswith('date')) for d in ncdimlist])[0]
#            existDim[3] = np.where([(d.lower().startswith('lev')) | (d.lower().startswith('dep')) for d in ncdimlist])[0]
        
        identified = existDim > -1
        
#        checkedDims[identified]=checkedDims[identified][existDim.compress(identified).astype(int)]
#        checkedDims=checkedDims[identified]
        
#        for cn, vn in enumerate(checkedDims) : dimStr.update({cn:len(ncf.dimensions[vn])})
        
        #Update dimension structure with identified dimensions
        #Load dimensional variables
        #TODO : Add scaling here in case...
        for i,d in enumerate(existDim) : 
            if identified[i] :
                dimStr.update({ncdimlist[d]:len(ncf.dimensions[ncdimlist[d]])}) #Append dimension
                cmd =  'load_ncVar(\'' + ncdimlist[d] + '\',nc=ncf)'
                self.message(4, 'loading : {0}={1}'.format(checkedDims[i],cmd))
                locals()[checkedDims[i]]=load_ncVar(ncdimlist[d], nc=ncf,**kwargs)

        missdims=set(ncdimlist)
        missdims.difference_update(ncdimlist[existDim[identified]])
        missdims=list(missdims)

        for i,d in enumerate(missdims) :
            dimStr.update({d:len(ncf.dimensions[d])})
            if ncf.variables.has_key(d) :
                cmd = 'load_ncVar(\'' + d + '\',nc=ncf)'
                self.message(4, 'loading : {0}={1}'.format(d,cmd))
                locals()[d]=load_ncVar(d, nc=ncf,**kwargs)
                        
            #If the variable associated to the dimension do not exist, generate it
            else :
                self.message(1, '[WARNING] Netcdf file not standard - creating data for {0} dimnsion'.format(d))
                ndim=len(ncf.dimensions[d])
                cmd = '=var'
                self.message(4, 'loading : {0}={1}'.format(d,cmd))
                locals()[d]={'_dimensions':{'_ndims':1,d:ndim}, 'data':np.arange(ndim)}
        
#        #Update dimension structure with identified dimensions
#        for cn, vn in zip(*(checkedDims[identified], ncdimlist[identified])) : dimStr.update({cn:len(ncf.dimensions[vn])})
#        for vn in ncdimlist[~identified] : dimStr.update({vn:len(ncf.dimensions[vn])})
        
        
        #Load dimensional variables
        #TODO : Add scaling here in case...
#        for cn, vn in zip(*(checkedDims[identified], ncdimlist[identified])) :
##            cmd = 'self.'+cn+'=ncf.variables[\''+vn+'\'][:]'
#            cmd = cn + '=load_ncVar(\'' + vn + '\',nc=ncf)'
#            self.message(4, 'exec : ' + cmd)
#            exec(cmd)
#            
#        for vn in ncdimlist[~identified] :
##            cmd = 'self.'+vn+'=ncf.variables[\''+vn+'\'][:]'
#            cmd = vn + '=load_ncVar(\'' + vn + '\',nc=ncf)'
#            self.message(4, 'exec : ' + cmd)
#            exec(cmd)
            
        #Update dimlist with dimensions present in the object
#        dimlist = np.append(checkedDims[identified], ncdimlist[~identified])

        dimlist=ncdimlist.copy()
#        dimlist[existDim[identified]]=checkedDims[identified]
        if identified.sum() > 0 : dimlist[existDim[identified]]=checkedDims[identified]
        else : dimlist = dimlist[[]]
        
#        for d in ncdimlist[checkedDims[identified]] :
#            if not d.startswith('_') : dimlist = np.append(dimlist,d)
#        dimlist=[(d if not d.startswith('_') else None) for d in dimStr.keys()]
        
        if params is not None :
            if force : par_list = [i.upper() for i in params]
            else :par_list = list(set(params).intersection(par_list))
        else : par_list = par_list.tolist()
        
        #remove dimensional  variable
        for d in ncdimlist[existDim[identified]] :
            par_list.pop(par_list.index(d))
        
        self.message(2, 'Recorded parameters : ' + str(nparam) + ' -> ' + str(par_list))
     
        
        #Extract within limits
        if (existDim[0] > -1) & (existDim[1] > -1):
            llind, flag = in_limits(lon['data'],lat['data'], limit=self.limit)
            if isinstance(flag,tuple) :
                lon['data'] = recale(lon['data'].compress(flag[0]),degrees=True)
                lon['_dimensions'][lon['_dimensions'].keys()[1]] = flag[0].sum()
                lat['data'] = lat['data'].compress(flag[1])
                lat['_dimensions'][lat['_dimensions'].keys()[1]] = flag[1].sum()
            else :
                lon['data'] = recale(lon['data'].compress(flag),degrees=True)
                lon['_dimensions'][lon['_dimensions'].keys()[1]] = flag.sum()
                lat['data'] = lat['data'].compress(flag)
                lat['_dimensions'][lat['_dimensions'].keys()[1]] = flag.sum()
            
            locals()[ncdimlist[existDim[0]]]=lon.copy()
            locals()[ncdimlist[existDim[1]]]=lat.copy()
            
            dimStr.update({ncdimlist[existDim[0]]:len(lon['data'])})
            dimStr.update({ncdimlist[existDim[1]]:len(lat['data'])})
#            self.message(4, 'self.lon & self.lat updated')
        
        if (existDim[2] > -1):
            if (timerange is not None) : timeflag = (time['data'] >= np.min(timerange)) & (time['data'] <= np.max(timerange))
            else : timeflag = np.ones(len(time['data']), dtype=bool)
            if timeflag.sum() == 0 : self.Error('No data within specified depth range (min/max = {0}/{1})'.format(np.min(time), np.max(time)))
            time['data'] = time['data'].compress(timeflag)
            time['_dimensions'][time['_dimensions'].keys()[1]] = timeflag.sum()
            locals()[ncdimlist[existDim[2]]]=time.copy()
            dimStr.update({ncdimlist[existDim[2]]:len(time['data'])})
#            self.message(4, 'self.lon & self.lat updated')
        
        #Extract within depth range
        if (existDim[3] > -1):
            if (depthrange is not None) : depthflag = (depth['data'] >= np.min(depthrange)) & (depth['data'] <= np.max(depthrange))
            else : depthflag = np.ones(len(depth['data']), dtype=bool)
            if depthflag.sum() == 0 : self.Error('No data within specified depth range (min/max = {0}/{1})'.format(np.min(depth), np.max(depth)))
            depth['data'] = depth['data'].compress(depthflag)
            depth['_dimensions'][depth['_dimensions'].keys()[1]] = depthflag.sum()
            locals()[ncdimlist[existDim[3]]]=depth.copy()
            dimStr.update({ncdimlist[existDim[3]]:len(depth['data'])})
        
        #Create output data structure
        outStr = OrderedDict()
        outStr.update({'_dimensions':dimStr})
        outStr.update({'_attributes':attrStr})
        
        if (existDim[0] > -1) : outStr.update({ncdimlist[existDim[0]]:lon})
        if (existDim[1] > -1) : outStr.update({ncdimlist[existDim[1]]:lat})
        if (existDim[2] > -1) : outStr.update({ncdimlist[existDim[2]]:time})
        if (existDim[3] > -1) : outStr.update({ncdimlist[existDim[3]]:depth})
        
        
        #Update object with remaining variables
        for d in dimlist.compress([not outStr.has_key(f) for f in dimlist]) :
#            cmd = 'outStr.update({\''+d+'\':'+d+'[\'data\']})'
            cmd = 'outStr.update({\''+d+'\':'+d+'})'
            self.message(4, 'exec : '+cmd)
            exec(cmd)
        
        ncdimStr=outStr.copy()
        #Get dimension lengths
        shape=()
        for d in dimlist: shape += np.shape(locals()[d]['data'])
        
        ndims = np.size(shape)
     
#        #Create dimension structure
#        curDim = [str(dimname) for dimname in dimStr.keys()[1:]] #[str(dimname) for dimname in ncf.variables['LONGITUDE'].dimensions]
#        curDimval = [dimStr[dim] for dim in curDim] #[len(ncf.dimensions[dimname]) for dimname in curDim]
#   
#        outStr={'_dimensions':{'_ndims':ndims,'nbpoints':sz[0]},'lon':lon,'lat':lat,'date':date}
        
#        for d in dimlist : outStr.update({d:self.__dict__[d]})

        #Sort NCDIMLIST to match DIMLIST
#        ncdimlist[np.sort(existDim.astype(np.int)[identified])]=ncdimlist[existDim[identified].tolist()]

        #Setup kwargs with current dimensionnal properties
        for d, ncd in zip(*(dimlist,ncdimlist)):
            if not kwargs.has_key(ncd) :
                if kwargs.has_key(d) :
                    
                    kwargs.update({ncd:kwargs[d]})
                    del kwargs[d]
                else :
                    dvar=ncdimStr[d]['data']
                    if isinstance(dvar,np.ma.masked_array) : kwargs.update({ncd:(np.nanmin(dvar.data),np.nanmax(dvar.data))})
                    else : kwargs.update({ncd:(np.nanmin(dvar),np.nanmax(dvar))})
#            else :
#                outStr['NbLatitudes']['data']        
        for param in par_list :
#            dumVar = load_ncVar(param,  nc=ncf, lon=llind[0], lat=llind[1], time=np.arange(len(time['data'])).compress(timeflag),**kwargs) #Load variables
#            dumVar = load_ncVar(param,  nc=ncf, longitude=(self.limit[1],self.limit[3]), latitude=(self.limit[0],self.limit[2]), time=(self.time.min(),self.time.max()),**kwargs) #Load variables
            
            
            dumVar = load_ncVar(param,  nc=ncf, **kwargs) #Load variables
#            dimStr = dumVar['_dimensions']
            
            #update dimensions
#            curDim = [str(dimname) for dimname in dimStr.keys()[1:]] #[str(dimname) for dimname in ncf.variables['LONGITUDE'].dimensions]
#            curDimval = [dimStr[dim] for dim in curDim] #[len(ncf.dimensions[dimname]) for dimname in curDim]
            
#            curDim = dimlist[where_list(curDim, ncdimlist.tolist())] #Convert to object dimension names
#            curDim = dimlist[where_list(curDim, dimlist.tolist())] #Convert to object dimension names (???)
            
##            curDim = [str(dimname) for dimname in ncf.variables[param].dimensions]
##            curDimval = [len(ncf.dimensions[dimname]) for dimname in curDim]
#            flag = [(np.array(dimname) == outStr['_dimensions'].keys()).sum() == 0 for dimname in curDim] #find dimensions to update
#            dimUpdate = np.array(curDim).compress(flag)
#            for enum in enumerate(dimUpdate) : 
#                self.message(2, 'Appending dimensions {0}:{1} to dataStructure'.format(enum[1], np.array(curDimval).compress(flag)[enum[0]]))
#                outStr['_dimensions'].update({enum[1]:np.array(curDimval).compress(flag)[enum[0]]}) #Append new dimension
#                outStr['_dimensions']['_ndims'] += 1 #update dimension counts
            
#            cmd = 'dumStr = {\'' + param + '\':dumVar[\'data\']}'


            #Set list as variable with attributes
#            if (not output_is_dict):
#                var=dumVar.pop('data')
#                for k in dumVar.keys():
#                    setattr(var, k, dumVar.pop(k))
#                dumVar=var.copy()
            
            cmd = 'dumStr = {\'' + param + '\':dumVar}'
            self.message(4, 'exec : ' + cmd)
            exec(cmd)
            outStr.update(dumStr)
            
            #Update output dimensions with extracted dimensions
            for ddum in dumStr[param]['_dimensions'].keys()[1:] :
                if outStr['_dimensions'].get(ddum) != dumStr[param]['_dimensions'][ddum] : outStr['_dimensions'][ddum]=dumStr[param]['_dimensions'][ddum]
            
#            cmd = 'self.'+param+'='
        
        ncf.close()
        return outStr
    
    def message(self, MSG_LEVEL, str):
        """
        print function wrapper. Print a message depending on the verbose level
        
        :param {in}{required}{type=int} MSG_LEVEL: level of the message to be compared with self.verbose
        
        :example: display a message
           
           .. code-block:: python
           
              self.log(0,'This message will be shown for any verbose level')
           
        :author: Renaud DUSSURGET (RD), LER PAC/IFREMER
        :change: Added a case for variables with missing dimensions
         
        """
        caller=get_caller()
        if MSG_LEVEL <= self.verbose : print('[{0}.{1}()] {2}'.format(__name__,caller.co_name,str))
        
    def Error(self, ErrorMsg):    
        raise Exception(ErrorMsg)
    
    def attributes(self, filename, **kwargs):
        """
        Get attributes of a NetCDF file
        
        :return {type:dict} outStr: Attribute structure.
        :author: Renaud Dussurget
        """
        
        #Open file
        self._filename = filename
        ncf = ncfile(self._filename, "r")
     
        #Get list of recorded parameters:
        keys = ncf.__dict__.keys()
        outStr = OrderedDict()
        for a in keys: outStr.update({a:ncf.__getattr__(a)})

        return outStr
        

def load_ncVar(varName, nc=None, **kwargs):
        '''
        Loads a variable from the NetCDF file and saves it as a data structure.
        
        :parameter varName: variable name
        :keywords kwargs: additional keyword arguments for slicing the dataset. Keywords should be named the name of the dimensions to subsample along and associated value should be a length 2 or 3 tuple (min,max,<step>).
        
        .. note: slices are provided in this interval : [min,max] (ie. including both extremities)
        
        '''
        
        if (nc is None) : raise Exception('No Netcdf file passed')
        
        var = nc.variables[varName]
        
        var.set_auto_maskandscale(False)
        
        #Load dimensions
        varDim = [str(dim) for dim in var.dimensions]
        missDim=len(varDim) == 0
        if (missDim):
            warn('No dimension found - creating it')
            sh=var[:].shape
            varDimval = sh
            varDim = ['dim_%02i' % (i+1) for i in xrange(len(varDimval))]
        else : varDimval = [len(nc.dimensions[dimname]) for dimname in varDim]
        
        #Load Attributes
        attrStr=var.__dict__
        
        ind_list = [] #Init index list
        dims = OrderedDict({'_ndims':0}) #Init dimensions
        
        dstr=[]
        shape=()

        #Construct index list
        #looping on variable dimension list
        for vid,vn in enumerate(varDim) :
            
            #No indexation on current dimension
            if not kwargs.has_key(vn) :
                dstr=np.append(dstr,':')
                sz=np.long(varDimval[vid])
#                ind_list.append(range(varDimval[vid])) # if no restriction in kargs then equivalent to [:]
#                dims.update({vn:varDimval[vid]})
            
            #Data is indexed along current dimension
            else :
                drange=kwargs[vn]
                if len(drange) == 2 : drange = drange + (1,)
                if nc.variables.has_key(vn) : #Check if current dimension exists
                    dumvar = nc.variables[vn][:]
                else :
                    dumvar = np.arange(len(nc.dimensions[vn]))
                if vn.startswith('lon') : dumvar=recale(dumvar,degrees=True)
                fg=(dumvar >= drange[0]) & (dumvar <= drange[1])
                if fg.sum() == 0 :
                    #retry switrhcing lon/lat
                    dumvar=recale(dumvar,degrees=True)
                    drange=tuple(recale(drange,degrees=True))#.astype(np.long))
                    fg=(dumvar >= drange[0]) & (dumvar <= drange[1])
                if fg.sum() == 0 :
                    raise IndexError('{0} {1} is not matching given dimensions {2}'.format(vn,(np.nanmin(nc.variables[vn][:]),np.nanmax(nc.variables[vn][:])),drange))
                if len(fg) == 1 :
                    dstr=np.append(dstr,':')
                    sz=1L
                elif len(fg) == 0:
                    sz=0L
                else :
                    dumind=np.arange(varDimval[vid],dtype=long).compress(fg)
                    bg=dumind[0]
                    en=dumind[-1]+1
                    st=long(drange[2])
                    dstr=np.append(dstr,'{0}:{1}:{2}'.format(bg,en,st))
                    sz = np.long(np.mod(np.float(en-bg-1)/st,np.float(en-bg)) + 1.)
            
            dims.update({vn:sz})
            shape = shape + (sz,)
#                if isinstance(dumind, np.ndarray) : dumind = dumind.tolist() #Rq: tolist() can take a very long time to run on large arrays
#                if type(dumind) is not list : dumind = [dumind] 
#                ind_list.append(dumind)
#                dims.update({vn:len(dumind)})
        
#        #check index list
#        sz = [np.size(i) for i in ind_list]
        
        dstr=','.join(dstr) #invert dimension list for numpy
#        dstr=','.join(dstr[::-1]) #invert dimension list for numpy
        if missDim : cmd = 'varOut = var[:]'
        else : cmd = 'varOut = var[{0}]'.format(dstr)
        exec(cmd)
        
        #find empty variables
#        if not (atools.where_list([0], shape)[0] == -1) : varOut = var[[0]][[]]
#        else : varOut = var[ind_list]
        
        #Mask it!
        if var.__dict__.has_key('_FillValue') :
            fill_value=var._FillValue
            mask = varOut == var._FillValue
        elif var.__dict__.has_key('missing_value') :
            fill_value=var._FillValue
            mask = varOut == var._FillValue
        else :
            fill_value = None
            mask = np.zeros(varOut.shape, dtype='bool')
        
        #Scale it
        #note : we do not use the *= or += operators to force casting to scaling attribute types
        if var.__dict__.has_key('scale') : varOut = varOut * var.scale
        elif var.__dict__.has_key('scale_factor') : varOut = varOut * var.scale_factor
        if var.__dict__.has_key('add_offset') : varOut = varOut + var.add_offset
        
        #Set masks properly
        if isinstance(varOut, np.ndarray) : varOut = np.ma.masked_array(varOut, mask=mask,dtype=varOut.dtype,fill_value=fill_value)
        elif isinstance(varOut, np.ma.masked_array) : var.mask = mask
        elif np.isscalar(varOut) : varOut = np.ma.masked_array([varOut], mask=mask,dtype=varOut.dtype,fill_value=fill_value) #Case of a scalar: cast to array and force having a shape
        else :
			try: varOut = np.ma.masked_array(np.array(varOut), mask=np.array(mask),dtype=varOut.dtype,fill_value=fill_value)   
			except: raise Exception('This data type (%s) has not been defined - code it!' % type(varOut))
        
        #Update masked data properly
        varOut.data[varOut.mask]=varOut.fill_value
        
        #Switch dimensions 
        if not missDim : varOut=np.transpose(varOut,tuple(range(len(dims.keys()[1:]))[::-1]))
        
        #Build up output structure
        dims.update({'_ndims':len(dims.keys()[1:])})
        outStr = {'_dimensions':dims, 'data':varOut}
        
        #Add variable attributes
        for A in var.__dict__.keys():
            outStr[A]=var.getncattr(A)
        
        return outStr
#            ind_list=[[]] 
    
#Additional functions
def load_ncVar_v2(varName,nc=None,**kwargs):
        
        if nc is None : raise 'No Netcdf file passed'
        
        #Load variable
        var = nc.variables[varName]
        
        var.set_auto_maskandscale(False)
        
        #Load dimensions
        varDim = [str(dim) for dim in var.dimensions]
        varDimval = [len(nc.dimensions[dimname]) for dimname in varDim]
        
        
#        from collections import deque
        ind_list = [] #deque() #Init index list
        #dims={'_ndims':0} #Init dimensions
        dims=dimStr()

        #Construct index list
        #looping on variable dimension list
        for enum in enumerate(varDim) :
            
            #No indexation on current dimension
            if not kwargs.has_key(enum[1]) :
#                ind_list+=tuple(xrange(varDimval[enum[0]]))
                ind_list.append(xrange(varDimval[enum[0]]))
#                ind_list=(ind_list,xrange(varDimval[enum[0]])) if len(ind_list) != 0 else (xrange(varDimval[enum[0]]),)
#                .append(xrange(varDimval[enum[0]])) # if no restriction in kargs then equivalent to [:]
                dims.update({enum[1]:varDimval[enum[0]]})
            
            #Data is indexed along current dimension
            else :
                dumind = kwargs[enum[1]]
#                if not isinstance(dumind,list) : dumind=tuple(dumind)
                if isinstance(dumind,np.ndarray) : dumind=dumind.tolist() #Rq: tolist() can take a very long time to run on large arrays
                if type(dumind) is not list : dumind=[dumind] 
                ind_list.append(dumind)
#                ind_list=(ind_list,dumind)  if len(ind_list) != 0 else (dumind,)
                dims.update({enum[1]:len(dumind)})
        
        #check index list
        sz=[len(i) for i in ind_list]
        
        #find empty dimensions
        if not (where_list([0],sz)[0] == -1 ) : varOut=var[[0]][[]] #np.array(where_list(sz,[0])) == -1
        else :
            varOut=var[ind_list]#.copy() #THIS IS LONG!!
            if var.shape == (1,1) : varOut=varOut.reshape(var.shape)
        
        #Mask it!
        if var.__dict__.has_key('_FillValue') : mask=varOut == var._FillValue
        elif var.__dict__.has_key('missing_value') : mask=varOut == var.missing_value
        else : mask=np.zeros(varOut.shape,dtype='bool')
        
        #Scale it
        #note : we do not use the *= or += operators to force casting to scaling attribute types
        if var.__dict__.has_key('scale') : varOut =varOut * var.scale
        elif var.__dict__.has_key('scale_factor') : varOut = varOut * var.scale_factor
        if var.__dict__.has_key('add_offset') : varOut = varOut + var.add_offset
        
        #Set masks properly
        if isinstance(varOut,np.ndarray) : varOut=np.ma.masked_array(varOut,mask=mask)
        elif isinstance(varOut,np.ma.masked_array) : var.mask=mask
        else : raise 'This data type {} has not been defined - code it!'.format(type(varOut))
        
        #Get attributes
        attr=attrStr(var.__dict__.keys(),var.__dict__.values())
        attr.pop('_FillValue',None) #Remove this attributed as it is overidden
        
        #Append attributes to varOut
        varOut.__dict__.update(attr)
        
        #Build up output structure
        outStr=varStr(dimensions=dims,
                      attributes=attr,
                      data=varOut)
        
#         outStr={'_dimensions':dims,'data':varOut}
#         outStr.update({'_attributes':attrStr})
#         dims.update({'_ndims':len(dims.keys()[1:])})
        
        return outStr
##            ind_list=[[]] 
